import os,sys,json
import unittest

test_dir = os.path.abspath(os.path.join(os.path.dirname(__file__)))
sys.path.insert(0, os.path.join(test_dir, '..'))
__package__ = 'tools'
from .context import stat_sys

class TestRunReport(unittest.TestCase):
    def test_stat_sys (self):
        folder = 'run_report_test_output'
        sys, sys_count, sys_all = stat_sys(os.path.join(test_dir,folder), verbose = False, mute = True)
        with open(os.path.join(test_dir, folder, 'param.json')) as fp:
            jdata = json.load(fp)
        self.assertEqual(sys, jdata['sys_configs'])
        self.assertEqual(sys_count, [jdata['fp_task_max'], jdata['fp_task_max']])
        ref_all = [[['npt', 50.0, 1.0, 4], ['npt', 50.0, 2.0, 1], ['npt', 100.0, 1.0, 2], ['npt', 100.0, 2.0, 1]], [['npt', 50.0, 1.0, 2], ['npt', 50.0, 2.0, 4], ['npt', 100.0, 1.0, 2]]]
        self.assertEqual(sys_all, ref_all)



#!/usr/bin/env python3

import os, re, shutil, logging

iter_format = "%s"
task_format = "%s"
log_iter_head = "task type: " + iter_format + " task: " + task_format + " process: "

def make_iter_name (iter_index) :
    return "task type:" + (iter_format)

def create_path (path) :
    path += '/'
    if os.path.isdir(path) : 
        dirname = os.path.dirname(path)        
        counter = 0
        while True :
            bk_dirname = dirname + ".bk%03d" % counter
            if not os.path.isdir(bk_dirname) : 
                shutil.move (dirname, bk_dirname) 
                break
            counter += 1
    os.makedirs (path)

def replace (file_name, pattern, subst) :
    file_handel = open (file_name, 'r')
    file_string = file_handel.read ()
    file_handel.close ()
    file_string = ( re.sub (pattern, subst, file_string) )
    file_handel = open (file_name, 'w')
    file_handel.write (file_string)
    file_handel.close ()

def copy_file_list (file_list, from_path, to_path) :
    for jj in file_list : 
        if os.path.isfile(os.path.join(from_path, jj)) :
            shutil.copy (os.path.join(from_path, jj), to_path)
        elif os.path.isdir(os.path.join(from_path, jj)) :
            shutil.copytree (os.path.join(from_path, jj), os.path.join(to_path, jj))

def cmd_append_log (cmd,
                    log_file) :
    ret = cmd
    ret = ret + " 1> " + log_file
    ret = ret + " 2> " + log_file
    return ret

def log_iter (task, ii, jj) :
    logging.info ((log_iter_head + "%s") % (ii, jj, task))

def repeat_to_length(string_to_expand, length):
    ret = ""
    for ii in range (length) : 
        ret += string_to_expand
    return ret

def log_task (message) :
    header = repeat_to_length (" ", len(log_iter_head % (0, 0)))
    logging.info (header + message)

def record_iter (record, confs, ii, jj) :
    with open (record, "a") as frec :
        frec.write ("%s %s %s\n" % (confs, ii, jj)) 



#!/usr/bin/python3 

import os
import numpy as np

def _reciprocal_box(box) :
    rbox = np.linalg.inv(box)
    rbox = rbox.T
    return rbox

def _make_pwmat_kp_mp(kpoints) :
    ret = ""
    ret += "%d %d %d 0 0 0 " % (kpoints[0], kpoints[1], kpoints[2])
    return ret

def _make_kspacing_kpoints(config, kspacing) :
    with open(config, 'r') as fp:
        lines = fp.read().split('\n')
    box = []
    for idx, ii in enumerate(lines):
        if 'lattice' in ii or 'Lattice' in ii or 'LATTICE' in ii:
            for kk in range(idx+1,idx+1+3):
                vector=[float(jj) for jj in lines[kk].split()[0:3]]
                box.append(vector)
            box = np.array(box)
            rbox = _reciprocal_box(box)
    kpoints = [(np.ceil(2 * np.pi * np.linalg.norm(ii) / kspacing).astype(int)) for ii in rbox]
    ret = _make_pwmat_kp_mp(kpoints)
    return ret


def make_pwmat_input_dict (node1, node2, atom_config, ecut, e_error, 
                            rho_error, icmix = None, smearing = None,
                            sigma = None,kspacing = 0.5, flag_symm = None) :
    input_dict = {}
    input_dict['node1'] = node1
    input_dict['node2'] = node2
    input_dict['in.atom'] = atom_config
    input_dict['ecut'] = ecut
    input_dict['e_error'] = e_error
    input_dict['rho_error'] = rho_error
    if icmix is not None:
        if sigma is not None:
            if smearing is not None:
                SCF_ITER0_1 = "6 4 3 0.0000 " + str(sigma) + " " + str(smearing)
                SCF_ITER0_2 = "94 4 3 " + str(icmix) + " " + str(sigma) + " " + str(smearing)
            else:
                SCF_ITER0_1 = "6 4 3 0.0000 " + str(simga) + " 2" 
                SCF_ITER0_2 = "94 4 3 " + str(icmix) + " " + str(simga) + " 2" 

        else:
            if smearing is not None:
                SCF_ITER0_1 = "6 4 3 0.0000 0.025 " + str(smearing)
                SCF_ITER0_2 = "94 4 3 " + str(icmix) + " 0.025 " + str(smearing)
            else:
                SCF_ITER0_1 = "6 4 3 0.0000 0.025 2" 
                SCF_ITER0_2 = "94 4 3 " + str(icmix) + " 0.025 2" 
    else:
        if sigma is not None:
            if smearing is not None:
                SCF_ITER0_1 = "6 4 3 0.0000 " + str(sigma) + " " + str(smearing)
                SCF_ITER0_2 = "94 4 3 1.0000 " + str(sigma) + " " + str(smearing)
            else:
                SCF_ITER0_1 = "6 4 3 0.0000 " + str(sigma) + " 2"
                SCF_ITER0_2 = "94 4 3 1.0000 " + str(sigma) + " 2"
        else:
            if smearing is not None:
                SCF_ITER0_1 = "6 4 3 0.0000 0.025 " + str(smearing)
                SCF_ITER0_2 = "94 4 3 1.0000 0.025 " + str(smearing)
            else:
                SCF_ITER0_1 = "6 4 3 0.0000 0.025 2"
                SCF_ITER0_2 = "94 4 3 1.0000 0.025 2"
    input_dict['scf_iter0_1'] = SCF_ITER0_1
    input_dict['scf_iter0_2'] = SCF_ITER0_2
    if flag_symm is not None :
        MP_N123 = _make_kspacing_kpoints(atom_config, kspacing)
        MP_N123 += str(flag_symm)
    else:
        MP_N123 = _make_kspacing_kpoints(atom_config, kspacing)
    input_dict['mp_n123'] = MP_N123
    input_dict['out.wg'] = 'F'
    input_dict['out.rho'] = 'F'
    input_dict['out.mlmd'] = 'T\n'
    return input_dict

def _update_input_dict(input_dict_, user_dict) :
    if user_dict is None:
        return input_dict_
    input_dict = input_dict_
    for ii in user_dict :
        input_dict[ci] = user_dict[ii]
    return input_dict

def write_input_dict(input_dict) :
    lines = []
    for key in input_dict:
        if (type(input_dict[key]) == bool):
            if input_dict[key]:
                rs = 'T'
            else :
                rs = 'F'
        else :
            rs = str(input_dict[key])
        lines.append('%s=%s' % (key, rs))
    return '\n'.join(lines)


def _make_smearing(fp_params) :
    icmix = None
    smearing = None
    sigma = None
    if 'icmix' in fp_params :
        icmix = fp_params['icmix']
    if 'smearing' in fp_params :
        smearing = fp_params['smearing']
    if 'sigma' in fp_params :
        sigma = fp_params['sigma']
    if icmix == None:
        if smearing == None:
            if sigma == None:
                return None, None, None
            else:
                return None, None, sigma
        else:
            if sigma == None:
                return None, smearing, None
            else:
                return None, smearing, sigma
    else:
        if smearing == None:
            if sigma == None:
                return icmix, None, None
            else:
                return icmix, None, sigma
        else:
            if sigma == None:
                return icmix, smearing, None
            else:
                return icmix, smearing, sigma
def _make_flag_symm(fp_params) :
    flag_symm = None
    if 'flag_symm' in fp_params :
        flag_symm = fp_params['flag_symm']
    if flag_symm == 'NONE' :
        flag_symm = None
    elif str(flag_symm) not in [None, '0', '1', '2', '3'] :
        raise RuntimeError ("unknow flag_symm type " + str(flag_symm))
    return flag_symm

def make_pwmat_input_user_dict(fp_params) :
    node1 = fp_params['node1']
    node2 = fp_params['node2']
    atom_config = fp_params['in.atom']
    ecut = fp_params['ecut']
    e_error = fp_params['e_error']
    rho_error = fp_params['rho_error']
    kspacing = fp_params['kspacing']
    if 'user_pwmat_params' in fp_params :
        user_dict = fp_params['user_pwmat_params']
    else :
        user_dict = None
    icmix, smearing, sigma = _make_smearing(fp_params)
    flag_symm = _make_flag_symm(fp_params)
    input_dict = make_pwmat_input_dict(node1, node2, atom_config, ecut, e_error,
                                        rho_error, icmix = icmix, smearing = smearing,
                                        sigma = sigma, kspacing = kspacing, 
                                        flag_symm = flag_symm
    )
    input_dict = _update_input_dict(input_dict, user_dict)
    input = write_input_dict(input_dict)
    return input
    
def input_upper(dinput):
    standard_input={}
    for key,val in dinput.items():
        standard_input[key.upper()]=val
    return Input(standard_input)



from aliyunsdkecs.request.v20140526.DescribeInstancesRequest import DescribeInstancesRequest
from aliyunsdkcore.client import AcsClient
from aliyunsdkcore.acs_exception.exceptions import ClientException
from aliyunsdkcore.acs_exception.exceptions import ServerException
from aliyunsdkecs.request.v20140526.RunInstancesRequest import RunInstancesRequest
from aliyunsdkecs.request.v20140526.DeleteInstancesRequest import DeleteInstancesRequest
from aliyunsdkecs.request.v20140526.DescribeAutoProvisioningGroupInstancesRequest import DescribeAutoProvisioningGroupInstancesRequest
from aliyunsdkecs.request.v20140526.CreateAutoProvisioningGroupRequest import CreateAutoProvisioningGroupRequest
from aliyunsdkecs.request.v20140526.DeleteAutoProvisioningGroupRequest import DeleteAutoProvisioningGroupRequest
from aliyunsdkecs.request.v20140526.ModifyAutoProvisioningGroupRequest import ModifyAutoProvisioningGroupRequest
from aliyunsdkecs.request.v20140526.DeleteLaunchTemplateRequest import DeleteLaunchTemplateRequest
from aliyunsdkvpc.request.v20160428.DescribeVpcsRequest import DescribeVpcsRequest
from aliyunsdkecs.request.v20140526.DescribeLaunchTemplatesRequest import DescribeLaunchTemplatesRequest
from aliyunsdkecs.request.v20140526.CreateLaunchTemplateRequest import CreateLaunchTemplateRequest
from aliyunsdkecs.request.v20140526.DescribeImagesRequest import DescribeImagesRequest
from aliyunsdkecs.request.v20140526.DescribeSecurityGroupsRequest import DescribeSecurityGroupsRequest
import time, json, os, glob, string, random
from dpgen.dispatcher.Dispatcher import Dispatcher, _split_tasks, JobRecord
from dpgen.dispatcher.SSHContext import SSHSession
from os.path import join
from dpgen  import dlog
from hashlib import sha1

def manual_delete(stage):
    with open('machine-ali.json') as fp1:
        mdata = json.load(fp1)
        adata = mdata[stage][0]['machine']['ali_auth']
        mdata_resources = mdata[stage][0]['resources']
        mdata_machine = mdata[stage][0]['machine']
        ali = ALI(adata, mdata_resources, mdata_machine, 0)
        with open('apg_id.json', 'r') as fp2:
            apg_id = json.load(fp2)
            ali.apg_id = apg_id['apg_id']
            ali.delete_apg()

def manual_create(stage, machine_number):
    with open('machine-ali.json') as fp:
        mdata = json.load(fp)
        adata = mdata[stage][0]['machine']['ali_auth']
        mdata_resources = mdata[stage][0]['resources']
        mdata_machine = mdata[stage][0]['machine']
        ali = ALI(adata, mdata_resources, mdata_machine, machine_number)
        ali.create_ess()
        print(ali.ip_list)

class ALI():
    def __init__(self, adata, mdata_resources, mdata_machine, nchunks):
        self.ip_list = ["unalloc" for i in range(nchunks)]
        self.instance_list = ["unalloc" for i in range(nchunks)]
        self.dispatchers = [[None, "unalloc"] for i in range(nchunks)]
        self.job_handlers = ["unalloc" for i in range(nchunks)]
        self.task_chunks = None
        self.adata = adata
        self.apg_id = None
        self.template_id = None
        self.vsw_id = None
        self.regionID = adata["regionID"]
        self.client = AcsClient(adata["AccessKey_ID"], adata["AccessKey_Secret"], self.regionID)
        self.mdata_resources = mdata_resources
        self.mdata_machine = mdata_machine
        self.nchunks = nchunks

    def init(self, work_path, tasks, group_size):
        if self.check_restart(work_path, tasks, group_size):
            pass
        else:
            self.create_ess()
            self.make_dispatchers()

    def create_ess(self):
        img_id = self.get_image_id(self.adata["img_name"])
        sg_id, vpc_id = self.get_sg_vpc_id()
        self.template_id = self.create_template(img_id, sg_id, vpc_id)
        self.vsw_id = self.get_vsw_id(vpc_id)
        self.apg_id = self.create_apg()
        dlog.info("begin to create ess, please wait two minutes")
        time.sleep(120)
        new_server_list = self.describe_apg_instances()
        new_ip_list = self.get_ip(new_server_list)
        for ii in range(len(new_server_list)):
            profile = self.mdata_machine.copy()
            profile['hostname'] = new_ip_list[ii]
            profile['instance_id'] = new_server_list[ii]
            if self.check_server(profile):
                self.instance_list[ii] = new_server_list[ii]
                self.ip_list[ii] = new_ip_list[ii]

    def delete_apg(self):
        request = DeleteAutoProvisioningGroupRequest()
        request.set_accept_format('json')
        request.set_AutoProvisioningGroupId(self.apg_id)
        request.set_TerminateInstances(True)
        try:
            response = self.client.do_action_with_exception(request)
        except ServerException as e:
            dlog.info(e)
        except ClientException as e:
            dlog.info(e)

    def create_apg(self):
        request = CreateAutoProvisioningGroupRequest()
        request.set_accept_format('json')
        request.set_TotalTargetCapacity(str(self.nchunks))
        request.set_LaunchTemplateId(self.template_id)
        request.set_AutoProvisioningGroupName(self.adata["instance_name"] + ''.join(random.choice(string.ascii_uppercase) for _ in range(20)))
        request.set_AutoProvisioningGroupType("maintain")
        request.set_SpotAllocationStrategy("lowest-price")
        request.set_SpotInstanceInterruptionBehavior("terminate")
        request.set_SpotInstancePoolsToUseCount(1)
        request.set_ExcessCapacityTerminationPolicy("termination")
        request.set_TerminateInstances(True)
        request.set_PayAsYouGoTargetCapacity("0")
        request.set_SpotTargetCapacity(str(self.nchunks))
        config = self.generate_config()
        request.set_LaunchTemplateConfigs(config)
        try:
            response = self.client.do_action_with_exception(request)
            response = json.loads(response)
            with open('apg_id.json', 'w') as fp:
                json.dump({'apg_id': response["AutoProvisioningGroupId"]}, fp, indent=4)
            return response["AutoProvisioningGroupId"]
        except ServerException as e:
            dlog.info(e)
        except ClientException as e:
            dlog.info(e)
        
    def update_server_list(self):
        instance_list = self.describe_apg_instances()
        return list(set(instance_list) - set(self.instance_list))

    def describe_apg_instances(self):
        request = DescribeAutoProvisioningGroupInstancesRequest()
        request.set_accept_format('json')
        request.set_AutoProvisioningGroupId(self.apg_id)
        request.set_PageSize(100)
        iteration = self.nchunks // 100
        instance_list = []
        for i in range(iteration + 1):
            request.set_PageNumber(i+1)
            response = self.client.do_action_with_exception(request)
            response = json.loads(response)
            for ins in response["Instances"]["Instance"]:
                instance_list.append(ins["InstanceId"])
        return instance_list
        
    def generate_config(self):
        machine_config = self.adata["machine_type_price"]
        config = []
        for conf in machine_config:
            for vsw in self.vsw_id:
                tmp = {
                    "InstanceType": conf["machine_type"],
                    "MaxPrice": str(conf["price_limit"] * conf["numb"]),
                    "VSwitchId": vsw,
                    "WeightedCapacity": "1",
                    "Priority": str(conf["priority"])
                }
                config.append(tmp)
        return config

    def create_template(self, image_id, sg_id, vpc_id):
        request = CreateLaunchTemplateRequest()
        request.set_accept_format('json')
        request.set_LaunchTemplateName(''.join(random.choice(string.ascii_uppercase) for _ in range(20)))
        request.set_ImageId(image_id)
        request.set_ImageOwnerAlias("self")
        request.set_PasswordInherit(True)
        request.set_InstanceType("ecs.c6.large")
        request.set_InstanceName(self.adata["instance_name"])
        request.set_SecurityGroupId(sg_id)
        request.set_VpcId(vpc_id)
        request.set_SystemDiskCategory("cloud_efficiency")
        request.set_SystemDiskSize(40)
        request.set_IoOptimized("optimized")
        request.set_InstanceChargeType("PostPaid")
        request.set_NetworkType("vpc")
        request.set_SpotStrategy("SpotWithPriceLimit")
        request.set_SpotPriceLimit(100)
        try:
            response = self.client.do_action_with_exception(request)
            response = json.loads(response)
            return response["LaunchTemplateId"]
        except ServerException as e:
            dlog.info(e)
        except ClientException as e:
            dlog.info(e)
            
    def delete_template(self):
        request = DeleteLaunchTemplateRequest()
        request.set_accept_format('json')
        request.set_LaunchTemplateId(self.template_id)
        response = self.client.do_action_with_exception(request)
        
    def get_image_id(self, img_name):
        request = DescribeImagesRequest()
        request.set_accept_format('json')
        request.set_ImageOwnerAlias("self")
        response = self.client.do_action_with_exception(request)
        response = json.loads(response)
        for img in response["Images"]["Image"]:
            if img["ImageName"] == img_name:
                return img["ImageId"]

    def get_sg_vpc_id(self):
        request = DescribeSecurityGroupsRequest()
        request.set_accept_format('json')
        response = self.client.do_action_with_exception(request)
        response = json.loads(response)
        for sg in response["SecurityGroups"]["SecurityGroup"]:
            if sg["SecurityGroupName"] == "sg":
                return sg["SecurityGroupId"], sg["VpcId"]

    def get_vsw_id(self, vpc_id):
        request = DescribeVpcsRequest()
        request.set_accept_format('json')
        request.set_VpcId(vpc_id)
        response = self.client.do_action_with_exception(request)
        response = json.loads(response)
        for vpc in response["Vpcs"]["Vpc"]:
            if vpc["VpcId"] == vpc_id:
                return vpc["VSwitchIds"]["VSwitchId"]

    def change_apg_capasity(self, capasity):
        request = ModifyAutoProvisioningGroupRequest()
        request.set_accept_format('json')
        request.set_AutoProvisioningGroupId(self.apg_id)
        request.set_TotalTargetCapacity(str(capasity))
        request.set_SpotTargetCapacity(str(capasity))
        request.set_PayAsYouGoTargetCapacity("0")
        response = self.client.do_action_with_exception(request)

    def check_spot_callback(self, instance_id):
        request = DescribeInstancesRequest()
        request.set_accept_format('json')
        request.set_InstanceIds([instance_id])
        status = False
        try:
            response = self.client.do_action_with_exception(request)
            response = json.loads(response)
            if len(response["Instances"]["Instance"]) == 1 and "Recycling" in response["Instances"]["Instance"][0]["OperationLocks"]["LockReason"]:
                status = True
            if instance_id not in self.describe_apg_instances():
                status = True
        except:
            pass
        return status

    def check_restart(self, work_path, tasks, group_size):
        if os.path.exists('apg_id.json'):
            with open('apg_id.json') as fp:
                apg = json.load(fp)
                self.apg_id = apg["apg_id"] 
            self.task_chunks = _split_tasks(tasks, group_size)
            task_chunks_str = ['+'.join(ii) for ii in self.task_chunks]
            task_hashes = [sha1(ii.encode('utf-8')).hexdigest() for ii in task_chunks_str]
            nchunks = len(self.task_chunks)
            for ii in range(nchunks):
                fn = 'jr.%.06d.json' % ii
                if not os.path.exists(os.path.join(os.path.abspath(work_path), fn)):
                    pass
                else:
                    job_record = JobRecord(work_path, self.task_chunks[ii], fname = fn)
                    cur_chunk = self.task_chunks[ii]
                    cur_hash = task_hashes[ii]
                    if not job_record.check_finished(cur_hash): 
                        with open(os.path.join(work_path, fn)) as fp:
                            jr = json.load(fp)
                            ip = jr[cur_hash]['context'][3]
                            instance_id = jr[cur_hash]['context'][4]
                            profile = self.mdata_machine.copy()
                            profile['hostname'] = ip
                            profile['instance_id'] = instance_id
                            if self.check_server(profile):
                                disp = Dispatcher(profile, context_type='ssh', batch_type='shell', job_record='jr.%.06d.json' % ii)
                                self.dispatchers[ii] = [disp, "working"]
                                self.ip_list[ii] = ip
                                self.instance_list[ii] = instance_id
                            # spot callback
                            else:
                                os.remove(os.path.join(work_path, fn))
                    else:
                        self.ip_list[ii] = "finished"
                        self.instance_list[ii] = "finished"
                        self.dispatchers[ii] = [None, "finished"]
            return True
        else:
            self.task_chunks = _split_tasks(tasks, group_size)
            return False

    def get_ip(self, instance_list):
        request = DescribeInstancesRequest()
        request.set_accept_format('json')
        ip_list = []
        if len(instance_list) <= 10:
            for i in range(len(instance_list)):
                request.set_InstanceIds([instance_list[i]])
                response = self.client.do_action_with_exception(request)
                response = json.loads(response)
                ip_list.append(response["Instances"]["Instance"][0]["VpcAttributes"]["PrivateIpAddress"]['IpAddress'][0])
        else:
            iteration = len(instance_list) // 10
            for i in range(iteration):
                for j in range(10):
                    request.set_InstanceIds([instance_list[i*10+j]])
                    response = self.client.do_action_with_exception(request)
                    response = json.loads(response)
                    ip_list.append(response["Instances"]["Instance"][0]["VpcAttributes"]["PrivateIpAddress"]['IpAddress'][0])
            if len(instance_list) - iteration * 10 != 0:
                for j in range(len(instance_list) - iteration * 10):
                    request.set_InstanceIds([instance_list[iteration*10+j]])
                    response = self.client.do_action_with_exception(request)
                    response = json.loads(response)
                    ip_list.append(response["Instances"]["Instance"][0]["VpcAttributes"]["PrivateIpAddress"]['IpAddress'][0])
        return ip_list

    def get_finished_job_num(self):
        finished_num = 0
        for ii in range(len(self.dispatchers)):
            if self.dispatchers[ii][1] == "finished":
                finished_num += 1
        return finished_num

    def resubmission(self, machine_exception_num):
        if self.adata["img_name"] == "kit":
            new_ip_list = []
            try:
                new_server_list = self.update_server_list()
                new_ip_list = self.get_ip(new_server_list)
            except:
                pass
            for ii in range(self.nchunks):
                if len(new_ip_list) == 0:
                    break
                if self.dispatchers[ii][1] == "exception":
                    self.ip_list[ii] = new_ip_list.pop()
                    self.instance_list[ii] = new_server_list.pop()
                    profile = self.mdata_machine.copy()
                    profile["hostname"] = self.ip_list[ii]
                    profile["instance_id"] = self.instance_list[ii]
                    self.dispatchers[ii] = [Dispatcher(profile, context_type='ssh', batch_type='shell', job_record="jr.%.06d.json" % ii), "working"]
                    dlog.info(self.ip_list[ii])
                    job_handler = self.dispatchers[ii][0].submit_jobs(resources,
                                                                      command,
                                                                      work_path,
                                                                      self.task_chunks[ii],
                                                                      group_size,
                                                                      forward_common_files,
                                                                      forward_task_files,
                                                                      backward_task_files,
                                                                      forward_task_deference,
                                                                      outlog,
                                                                      errlog)
                    self.job_handlers[ii] = job_handler
        elif self.adata["img_name"] == "vasp":
            if machine_exception_num / self.nchunks > 0.05:
                self.change_apg_capasity(self.nchunks - self.get_finished_job_num() + machine_exception_num)
                time.sleep(120)
                new_ip_list = []
                try:
                    new_server_list = self.update_server_list()
                    new_ip_list = self.get_ip(new_server_list)
                except:
                    pass
                if len(new_ip_list) == machine_exception_num:
                    dlog.info("new submission of callback machine")
                    for ii in range(self.nchunks):
                        self.ip_list[ii] = new_ip_list.pop()
                        self.instance_list[ii] = new_server_list.pop()
                        profile = self.mdata_machine.copy()
                        profile["hostname"] = self.ip_list[ii]
                        profile["instance_id"] = self.instance_list[ii]
                        self.dispatchers[ii] = [Dispatcher(profile, context_type='ssh', batch_type='shell', job_record="jr.%.06d.json" % ii), "working"]
                        dlog.info(self.ip_list[ii])
                        job_handler = self.dispatchers[ii][0].submit_jobs(resources,
                                                                          command,
                                                                          work_path,
                                                                          self.task_chunks[ii],
                                                                          group_size,
                                                                          forward_common_files,
                                                                          forward_task_files,
                                                                          backward_task_files,
                                                                          forward_task_deference,
                                                                          outlog,
                                                                          errlog)
                        self.job_handlers[ii] = job_handler

    #@profile(precision=6)
    def run_jobs(self,
                 resources,
                 command,
                 work_path,
                 tasks,
                 group_size,
                 forward_common_files,
                 forward_task_files,
                 backward_task_files,
                 forward_task_deference = True,
                 mark_failure = False,
                 outlog = 'log',
                 errlog = 'err'):
        for ii in range(self.nchunks):
            if self.dispatchers[ii][1] == "working":
                dlog.info(self.ip_list[ii])
                job_handler = self.dispatchers[ii][0].submit_jobs(resources,
                                                               command,
                                                               work_path,
                                                               self.task_chunks[ii],
                                                               group_size,
                                                               forward_common_files,
                                                               forward_task_files,
                                                               backward_task_files,
                                                               forward_task_deference,
                                                               outlog,
                                                               errlog)
                self.job_handlers[ii] = job_handler
        machine_exception_num = 0
        while True:
            new_ip_list = []
            try:
                new_server_list = self.update_server_list()
                new_ip_list = self.get_ip(new_server_list)
            except:
                pass
            if machine_exception_num > 0:
                self.resubmission(machine_exception_num)
            for ii in range(self.nchunks):
                if self.dispatchers[ii][1] == "working":
                    if self.check_spot_callback(self.instance_list[ii]):
                        machine_exception_num += 1
                        dlog.info("machine %s callback" % self.instance_list[ii])
                        os.remove(self.job_handlers[ii]["job_record"].fname)
                        self.job_handlers[ii] = "exception"
                        self.ip_list[ii] = "exception"
                        self.instance_list[ii] = "exception"
                        self.dispatchers[ii][1] = "exception"
                        if self.adata["img_name"] == "vasp":
                            self.change_apg_capasity(self.nchunks - self.get_finished_job_num() - machine_exception_num)
                        continue
                    if self.check_server(self.dispatchers[ii][0].remote_profile):
                        if self.dispatchers[ii][0].all_finished(self.job_handlers[ii], mark_failure, False):
                            self.delete(ii)
                            self.dispatchers[ii][1] = "finished"
                            self.ip_list[ii] = "finished"
                            self.instance_list[ii] = "finished"
                            self.change_apg_capasity(self.nchunks - self.get_finished_job_num())
                    else:
                        dlog.info("ssh exception accured in %s" % self.ip_list[ii])
                elif self.dispatchers[ii][1] == "finished":
                    continue
                elif self.dispatchers[ii][1] == "unalloc":
                    if new_ip_list:
                        profile = self.mdata_machine.copy()
                        profile["hostname"] = new_ip_list[0]
                        profile["instance_id"] = new_server_list[0]
                        if self.check_server(profile):
                            self.ip_list[ii] = new_ip_list.pop(0)
                            self.instance_list[ii] = new_server_list.pop(0)
                            self.dispatchers[ii] = [Dispatcher(profile, context_type='ssh', batch_type='shell', job_record='jr.%.06d.json' % ii), "working"]
                            dlog.info(self.ip_list[ii])
                            job_handler = self.dispatchers[ii][0].submit_jobs(resources,
                                                                   command,
                                                                   work_path,
                                                                   self.task_chunks[ii],
                                                                   group_size,
                                                                   forward_common_files,
                                                                   forward_task_files,
                                                                   backward_task_files,
                                                                   forward_task_deference,
                                                                   outlog,
                                                                   errlog)
                            self.job_handlers[ii] = job_handler
            if self.check_dispatcher_finished():
                os.remove('apg_id.json')
                self.delete_template()
                self.delete_apg()
                break
            else:
                time.sleep(10)

# status = ["unalloc", "working", "finished", "exception"]
    def check_server(self, profile):
        try:
            session = SSHSession(profile)
            session.close()
            return True
        except:
            return False

    def check_dispatcher_finished(self):
        count = 0
        flag = True
        for ii in range(len(self.dispatchers)):
            if self.dispatchers[ii][1] == "unalloc" or self.dispatchers[ii][1] == "working": flag = False
            if self.dispatchers[ii][1] == "exception": count += 1
        if self.adata["img_name"] == "vasp" and count / self.nchunks > 0.05: flag = False
        elif self.adata["img_name"] == "kit" and count > 0: flag = False
        return flag
   
    def delete(self, ii):
        request = DeleteInstancesRequest()
        request.set_accept_format('json')
        request.set_InstanceIds([self.instance_list[ii]])
        request.set_Force(True)
        response = self.client.do_action_with_exception(request)

    def make_dispatchers(self):
        for ii in range(self.nchunks):
            if self.ip_list[ii] != "unalloc":
                profile = self.mdata_machine.copy()
                profile['hostname'] = self.ip_list[ii]
                profile['instance_id'] = self.instance_list[ii]
                if self.check_server(profile):
                    self.dispatchers[ii] = [Dispatcher(profile, context_type='ssh', batch_type='shell', job_record='jr.%.06d.json' % ii), "working"]
                else:
                    continue
                    #self.dispatchers[ii] = [Dispatcher(profile, context_type='ssh', batch_type='shell', job_record='jr.%.06d.json' % ii), "working"]

    def delete_machine(self):
        request = DeleteInstancesRequest()
        request.set_accept_format('json')
        if len(self.instance_list) <= 100:
            request.set_InstanceIds(self.instance_list)
            request.set_Force(True)
            response = self.client.do_action_with_exception(request)
        else:
            iteration = len(self.instance_list) // 100
            for i in range(iteration):
                request.set_InstanceIds(self.instance_list[i*100:(i+1)*100])
                request.set_Force(True)
                response = self.client.do_action_with_exception(request)
            if len(self.instance_list) - iteration * 100 != 0:
                request.set_InstanceIds(self.instance_list[iteration*100:])
                request.set_Force(True)
                response = self.client.do_action_with_exception(request)
        self.instance_list = []
        self.ip_list = []
        os.remove('machine_record.json')
        dlog.debug("Successfully free the machine!")



#!/usr/bin/env python3

import os, re, argparse, filecmp, json, glob, shutil
import subprocess as sp
import numpy as np
import dpgen.auto_test.lib.vasp as vasp
import dpgen.auto_test.lib.lammps as lammps

from dpgen import dlog
from dpgen.generator.lib.vasp import incar_upper
from pymatgen.core.structure import Structure
from pymatgen.io.vasp import Incar
from dpgen import ROOT_PATH

cvasp_file=os.path.join(ROOT_PATH,'generator/lib/cvasp.py')

global_equi_name = '00.equi'
global_task_name = '01.eos'

'''
link poscar
link potcar
make incar
'''
def make_vasp(jdata, conf_dir) :
    vol_start = jdata['vol_start']
    vol_end = jdata['vol_end']
    vol_step = jdata['vol_step']
    eos_relax_cell_shape = jdata.get('eos_relax_cell_shape', True)
    conf_path = os.path.abspath(conf_dir)
    conf_poscar = os.path.join(conf_path, 'POSCAR')

    if 'relax_incar' in jdata.keys():
        vasp_str='vasp-relax_incar'
    else:
        kspacing = jdata['vasp_params']['kspacing']
        vasp_str='vasp-k%.2f' % kspacing

    # get equi poscar
    equi_path = re.sub('confs', global_equi_name, conf_path)
    equi_path = os.path.join(equi_path, vasp_str)
    equi_contcar = os.path.join(equi_path, 'CONTCAR')
    task_path = re.sub('confs', global_task_name, conf_path)
    task_path = os.path.join(task_path, vasp_str)
    os.makedirs(task_path, exist_ok = True)    
    # link poscar
    cwd = os.getcwd()
    from_poscar = os.path.join(equi_contcar)
    to_poscar = os.path.join(task_path, 'POSCAR')
    if os.path.exists(to_poscar) :
        assert(filecmp.cmp(from_poscar, to_poscar))
    else :
        os.chdir(task_path)
        os.symlink(os.path.relpath(from_poscar), 'POSCAR')
        os.chdir(cwd)
    vol_to_poscar = vasp.poscar_vol(to_poscar) / vasp.poscar_natoms(to_poscar)
    # print(to_poscar, vol_to_poscar)
    is_alloy = \
               os.path.exists(
                   os.path.join(
                       os.path.join(conf_path, '..'),
                       'alloy'
                   )
               )  
    # read potcar
    with open(to_poscar,'r') as fp :
        lines = fp.read().split('\n')
        ele_list = lines[5].split()
    potcar_map = jdata['potcar_map']
    potcar_list = []
    for ii in ele_list :
        assert os.path.exists(os.path.abspath(potcar_map[ii])),"No POTCAR in the potcar_map of %s"%(ii)
        potcar_list.append(os.path.abspath(potcar_map[ii]))

    # gen incar
    if  'relax_incar' in jdata.keys():
        relax_incar_path = jdata['relax_incar']
        assert(os.path.exists(relax_incar_path))
        relax_incar_path = os.path.abspath(relax_incar_path)
        incar = incar_upper(Incar.from_file(relax_incar_path))
        if eos_relax_cell_shape:
            isif = 4
        else:
            isif = 2
        if incar.get('ISIF') != isif:
            dlog.info("%s:%s setting ISIF to %d" % (__file__, make_vasp.__name__, isif))
            incar['ISIF'] = isif
        fc = incar.get_string()
        kspacing = incar['KSPACING']
        kgamma = incar['KGAMMA']
    else :
        fp_params = jdata['vasp_params']
        ecut = fp_params['ecut']
        ediff = fp_params['ediff']
        npar = fp_params['npar']
        kpar = fp_params['kpar']
        kspacing = fp_params['kspacing']
        kgamma = fp_params['kgamma']
        fc = vasp.make_vasp_relax_incar(ecut, ediff, is_alloy,  True, False, npar, kpar, kspacing, kgamma)

    os.chdir(task_path)

    with open('INCAR', 'w') as fp :
        fp.write(fc)
    # gen potcar
    with open('POTCAR', 'w') as outfile:
        for fname in potcar_list:
            with open(fname) as infile:
                outfile.write(infile.read())
    # loop over volumes
    for vol in np.arange(vol_start, vol_end, vol_step) :
        vol_path = os.path.join(task_path, 'vol-%.2f' % vol)        
        os.makedirs(vol_path, exist_ok = True)
        os.chdir(vol_path)
        for ii in ['INCAR', 'POTCAR', 'POSCAR.orig', 'POSCAR'] :
            if os.path.exists(ii) :
                os.remove(ii)
        # link incar, potcar
        os.symlink(os.path.relpath(os.path.join(task_path, 'INCAR')), 'INCAR')
        os.symlink(os.path.relpath(os.path.join(task_path, 'POTCAR')), 'POTCAR')
        # gen poscar
        os.symlink(os.path.relpath(to_poscar), 'POSCAR.orig')
        scale = (vol / vol_to_poscar) ** (1./3.)
        # print(scale)
        vasp.poscar_scale('POSCAR.orig', 'POSCAR', scale)
        # print(vol_path, vasp.poscar_vol('POSCAR') / vasp.poscar_natoms('POSCAR'))
        # gen kpoints
        fc = vasp.make_kspacing_kpoints('POSCAR', kspacing, kgamma)
        with open('KPOINTS', 'w') as fp: fp.write(fc)
        #copy cvasp
        if ('cvasp' in jdata) and (jdata['cvasp'] == True):
           shutil.copyfile(cvasp_file, os.path.join(vol_path,'cvasp.py'))
        os.chdir(cwd)


def make_lammps (jdata, conf_dir,task_type) :
    fp_params = jdata['lammps_params']
    model_dir = fp_params['model_dir']
    type_map = fp_params['type_map'] 
    model_dir = os.path.abspath(model_dir)
    model_name =fp_params['model_name']
    deepmd_version = fp_params.get("deepmd_version", "0.12")
    if not model_name and task_type =='deepmd':
        models = glob.glob(os.path.join(model_dir, '*pb'))
        model_name = [os.path.basename(ii) for ii in models]
        assert len(model_name)>0,"No deepmd model in the model_dir"
    else:
        models = [os.path.join(model_dir,ii) for ii in model_name]

    model_param = {'model_name' :      model_name,
                  'param_type':          fp_params['model_param_type'],
                  'deepmd_version' : deepmd_version}
    ntypes = len(type_map)

    vol_start = jdata['vol_start']
    vol_end = jdata['vol_end']
    vol_step = jdata['vol_step']

    # # get equi props
    # equi_path = re.sub('confs', global_equi_name, conf_path)
    # equi_path = os.path.join(equi_path, 'lmp')
    # equi_log = os.path.join(equi_path, 'log.lammps')
    # if not os.path.isfile(equi_log) :
    #     raise RuntimeError("the system should be equilibriated first")
    # natoms, epa, vpa = lammps.get_nev(equi_log)
    # task path
    task_path = re.sub('confs', global_task_name, conf_dir)
    task_path = os.path.abspath(task_path)
    os.makedirs(task_path, exist_ok = True)
    cwd = os.getcwd()
    conf_path = os.path.abspath(conf_dir)
    from_poscar = os.path.join(conf_path, 'POSCAR')
    to_poscar = os.path.join(task_path, 'POSCAR')
    if os.path.exists(to_poscar) :
        assert(filecmp.cmp(from_poscar, to_poscar))
    else :
        os.chdir(task_path)
        os.symlink(os.path.relpath(from_poscar), 'POSCAR')
        os.chdir(cwd)
    volume = vasp.poscar_vol(to_poscar)
    natoms = vasp.poscar_natoms(to_poscar)
    vpa = volume / natoms
    # structrure
    ss = Structure.from_file(to_poscar)
    
    # lmp path
    lmp_path = os.path.join(task_path, task_type)
    os.makedirs(lmp_path, exist_ok = True)
    #Null lammps.in
    f_lammps_in = os.path.join(lmp_path, 'lammps.in')
    with open(f_lammps_in, 'w') as fp :
        None
    # # lmp conf
    # conf_file = os.path.join(lmp_path, 'conf.lmp')
    # lammps.cvt_lammps_conf(to_poscar, conf_file)
    # ptypes = vasp.get_poscar_types(to_poscar)
    # lammps.apply_type_map(conf_file, deepmd_type_map, ptypes)

    os.chdir(lmp_path)
    for ii in model_name :
        if os.path.exists(ii) :
            os.remove(ii)
    for (ii,jj) in zip(models, model_name) :
        os.symlink(os.path.relpath(ii), jj)
    share_models =[os.path.join(lmp_path,ii) for ii in model_name]

    for vol in np.arange(vol_start, vol_end, vol_step) :
        vol_path = os.path.join(lmp_path, 'vol-%.2f' % vol)        
        print('# generate %s' % (vol_path))
        os.makedirs(vol_path, exist_ok = True)
        os.chdir(vol_path)
        #print(vol_path)
        for ii in ['conf.lmp', 'conf.lmp'] + model_name :
            if os.path.exists(ii) :
                os.remove(ii)                
        # # link conf
        # os.symlink(os.path.relpath(conf_file), 'conf.lmp')
        # make conf
        scale_ss = ss.copy()
        scale_ss.scale_lattice(vol * natoms)
        scale_ss.to('POSCAR', 'POSCAR')
        lammps.cvt_lammps_conf('POSCAR', 'conf.lmp')
        ptypes = vasp.get_poscar_types('POSCAR')
        lammps.apply_type_map('conf.lmp', type_map, ptypes)
        # link models
        for (ii,jj) in zip(share_models, model_name) :
            os.symlink(os.path.relpath(ii), jj)
        # make lammps input
        scale = (vol / vpa) ** (1./3.)
        if task_type=='deepmd':
            fc = lammps.make_lammps_press_relax('conf.lmp', ntypes, scale,lammps.inter_deepmd, model_param)
        elif task_type =='meam':
            fc = lammps.make_lammps_press_relax('conf.lmp', ntypes, scale,lammps.inter_meam, model_param)    
        with open(os.path.join(vol_path, 'lammps.in'), 'w') as fp :
            fp.write(fc)
        os.chdir(cwd)

def make_lammps_fixv (jdata, conf_dir,task_type) :
    fp_params = jdata['lammps_params']
    model_dir = fp_params['model_dir']
    type_map = fp_params['type_map'] 
    model_dir = os.path.abspath(model_dir)
    model_name =fp_params['model_name']
    deepmd_version = fp_params.get("deepmd_version", "0.12")
    if not model_name and task_type =='deepmd':
        models = glob.glob(os.path.join(model_dir, '*pb'))
        model_name = [os.path.basename(ii) for ii in models]
    else:
        models = [os.path.join(model_dir,ii) for ii in model_name]

    model_param = {'model_name' :      model_name,
                  'param_type':          fp_params['model_param_type'],
                  'deepmd_version' : deepmd_version}
    ntypes = len(type_map)


    vol_start = jdata['vol_start']
    vol_end = jdata['vol_end']
    vol_step = jdata['vol_step']

    # get equi props
    equi_path = re.sub('confs', global_equi_name, conf_dir)
    task_path = re.sub('confs', global_task_name, conf_dir)
    equi_path = os.path.join(equi_path, task_type)
    task_path = os.path.join(task_path, task_type)
    equi_path = os.path.abspath(equi_path)
    task_path = os.path.abspath(task_path)
    equi_dump = os.path.join(equi_path, 'dump.relax')
    os.makedirs(task_path, exist_ok = True)
    task_poscar = os.path.join(task_path, 'POSCAR')
    lammps.poscar_from_last_dump(equi_dump, task_poscar, type_map)

    cwd = os.getcwd()
    volume = vasp.poscar_vol(task_poscar)
    natoms = vasp.poscar_natoms(task_poscar)
    vpa = volume / natoms
    # structrure
    ss = Structure.from_file(task_poscar)
    # make lammps.in
    if task_type=='deepmd':
        fc = lammps.make_lammps_equi('conf.lmp', 
                                 ntypes, 
                                 lammps.inter_deepmd, 
                                 model_param, 
                                 change_box = False)
    elif task_type=='meam':
        fc = lammps.make_lammps_equi('conf.lmp', 
                                 ntypes, 
                                 lammps.inter_meam, 
                                 model_param, 
                                 change_box = False)
    f_lammps_in = os.path.join(task_path, 'lammps.in')
    with open(f_lammps_in, 'w') as fp :
        fp.write(fc)

    os.chdir(task_path)
    for ii in model_name :
        if os.path.exists(ii) :
            os.remove(ii)
    for (ii,jj) in zip(models, model_name) :
        os.symlink(os.path.relpath(ii), jj)
    share_models = [os.path.join(task_path,ii) for ii in model_name]

    # make vols
    for vol in np.arange(vol_start, vol_end, vol_step) :
        vol_path = os.path.join(task_path, 'vol-%.2f' % vol)        
        print('# generate %s' % (vol_path))
        os.makedirs(vol_path, exist_ok = True)
        os.chdir(vol_path)
        for ii in ['conf.lmp', 'conf.lmp', 'lammps.in'] + model_name :
            if os.path.exists(ii) :
                os.remove(ii)                
        # make conf
        scale_ss = ss.copy()
        scale_ss.scale_lattice(vol * natoms)
        scale_ss.to('POSCAR', 'POSCAR')
        lammps.cvt_lammps_conf('POSCAR', 'conf.lmp')
        ptypes = vasp.get_poscar_types('POSCAR')
        lammps.apply_type_map('conf.lmp', type_map, ptypes)
        # link lammps.in
        os.symlink(os.path.relpath(f_lammps_in), 'lammps.in')
        # link models
        for (ii,jj) in zip(share_models,model_name) :
            os.symlink(os.path.relpath(ii), jj)
        # make lammps input
        os.chdir(cwd)


def _main() :
    parser = argparse.ArgumentParser(
        description="gen 01.eos")
    parser.add_argument('TASK', type=str,
                        help='the task of generation, vasp or lammps')
    parser.add_argument('PARAM', type=str,
                        help='json parameter file')
    parser.add_argument('CONF', type=str,
                        help='the path to conf')
    parser.add_argument('-f', '--fix-shape', action = 'store_true',
                        help='fix shape of box')
    args = parser.parse_args()

    with open (args.PARAM, 'r') as fp :
        jdata = json.load (fp)

    # print('generate %s task with conf %s' % (args.TASK, args.CONF))
    if args.TASK == 'vasp':
        make_vasp(jdata, args.CONF)               
    elif args.TASK == 'deepmd' or args.TASK =='meam':
        if args.fix_shape :
            make_lammps_fixv(jdata, args.CONF,args.TASK)
        else :
            make_lammps(jdata, args.CONF,args.TASK)              
    else :
        raise RuntimeError("unknow task ", args.TASK)
    
if __name__ == '__main__' :
    _main()




#!/usr/bin/env python3

import os, re, argparse, filecmp, json, glob, warnings, shutil
import subprocess as sp
import numpy as np
import dpgen.auto_test.lib.vasp as vasp
import dpgen.auto_test.lib.lammps as lammps
from pymatgen.core.structure import Structure
from pymatgen.analysis.defects.core import Interstitial
from pymatgen.analysis.defects.generators import InterstitialGenerator

from dpgen import ROOT_PATH
from pymatgen.io.vasp import Incar
from dpgen.generator.lib.vasp import incar_upper
cvasp_file=os.path.join(ROOT_PATH,'generator/lib/cvasp.py')


global_equi_name = '00.equi'
global_task_name = '04.interstitial'

def make_vasp(json, conf_dir, supercell, insert_ele) :
    for ii in insert_ele :
        _make_vasp(json, conf_dir, supercell, ii)

def _make_vasp(jdata, conf_dir, supercell, insert_ele) :

    conf_path = os.path.abspath(conf_dir)
    conf_poscar = os.path.join(conf_path, 'POSCAR')
    # get equi poscar
    if 'relax_incar' in jdata.keys():
        vasp_str='vasp-relax_incar'
    else:
        kspacing = jdata['vasp_params']['kspacing']
        vasp_str='vasp-k%.2f' % (kspacing)
    equi_path = re.sub('confs', global_equi_name, conf_path)
    equi_path = os.path.join(equi_path, vasp_str)
    equi_contcar = os.path.join(equi_path, 'CONTCAR')
    assert os.path.exists(equi_contcar),"Please compute the equilibrium state using vasp first"
    task_path = re.sub('confs', global_task_name, conf_path)
    task_path = os.path.join(task_path, vasp_str)
    os.makedirs(task_path, exist_ok=True)
    cwd = os.getcwd()
    os.chdir(task_path)
    if os.path.isfile('POSCAR') :
        os.remove('POSCAR')
    os.symlink(os.path.relpath(equi_contcar), 'POSCAR')
    os.chdir(cwd)
    task_poscar = os.path.join(task_path, 'POSCAR')
    # gen strcture
    print("task poscar: ", task_poscar)
    ss = Structure.from_file(task_poscar)
    # gen defects
    vds = InterstitialGenerator(ss, insert_ele)
    dss = []
    for jj in vds :
        dss.append(jj.generate_defect_structure(supercell))
    # gen incar
    if  'relax_incar' in jdata.keys():
        relax_incar_path = jdata['relax_incar']
        assert(os.path.exists(relax_incar_path))
        relax_incar_path = os.path.abspath(relax_incar_path)
        incar = incar_upper(Incar.from_file(relax_incar_path))
        fc = incar.get_string()
        kspacing = incar['KSPACING']
        kgamma = incar['KGAMMA']

    else :
        fp_params = jdata['vasp_params']
        ecut = fp_params['ecut']
        ediff = fp_params['ediff']
        npar = fp_params['npar']
        kpar = fp_params['kpar']
        kspacing = fp_params['kspacing']
        kgamma = fp_params['kgamma']
        fc = vasp.make_vasp_relax_incar(ecut, ediff, True, True, True, npar=npar,kpar=kpar, kspacing = kspacing, kgamma = kgamma)
    with open(os.path.join(task_path, 'INCAR'), 'w') as fp :
        fp.write(fc)
    # gen tasks
    copy_str = "%sx%sx%s" % (supercell[0], supercell[1], supercell[2])
    cwd = os.getcwd()
    for ii in range(len(dss)) :
        struct_path = os.path.join(task_path, 'struct-%s-%s-%03d' % (insert_ele,copy_str,ii))
        print('# generate %s' % (struct_path))
        os.makedirs(struct_path, exist_ok=True)
        os.chdir(struct_path)
        for jj in ['POSCAR', 'POTCAR', 'INCAR'] :
            if os.path.isfile(jj):
                os.remove(jj)
        # make conf
        dss[ii].to('POSCAR', 'POSCAR')
        # gen potcar
        with open('POSCAR','r') as fp :
            lines = fp.read().split('\n')
            ele_list = lines[5].split()

        os.chdir(cwd)
        potcar_map = jdata['potcar_map']
        potcar_list = []
        for ii in ele_list :
            assert os.path.exists(os.path.abspath(potcar_map[ii])),"No POTCAR in the potcar_map of %s"%(ii)
            potcar_list.append(os.path.abspath(potcar_map[ii]))
        os.chdir(struct_path)

        with open('POTCAR', 'w') as outfile:
            for fname in potcar_list:
                with open(fname) as infile:
                    outfile.write(infile.read())

        # link incar
        os.symlink(os.path.relpath(os.path.join(task_path, 'INCAR')), 'INCAR')
        # save supercell
        np.savetxt('supercell.out', supercell, fmt='%d')

        # write kp
        fc = vasp.make_kspacing_kpoints('POSCAR', kspacing, kgamma)
        with open('KPOINTS', 'w') as fp: fp.write(fc)

        #copy cvasp
        if ('cvasp' in jdata) and (jdata['cvasp'] == True):
           shutil.copyfile(cvasp_file, os.path.join(struct_path,'cvasp.py'))
    os.chdir(cwd)


def make_reprod_traj(jdata, conf_dir, supercell, insert_ele, task_type) :
    for ii in insert_ele :
        _make_reprod_traj(jdata, conf_dir, supercell, ii, task_type)

def _make_reprod_traj(jdata, conf_dir, supercell, insert_ele, task_type) :

    fp_params = jdata['lammps_params']
    model_dir = fp_params['model_dir']
    type_map = fp_params['type_map']
    model_dir = os.path.abspath(model_dir)
    model_name =fp_params['model_name']
    deepmd_version = fp_params.get("deepmd_version", "0.12")
    if not model_name and task_type=='deepmd':
        models = glob.glob(os.path.join(model_dir, '*pb'))
        model_name = [os.path.basename(ii) for ii in models]
        assert len(model_name)>0,"No deepmd model in the model_dir"
    else:
        models = [os.path.join(model_dir,ii) for ii in model_name]

    model_param = {'model_name' :      model_name,
                  'param_type':          fp_params['model_param_type'],
                  'deepmd_version' : deepmd_version}

    ntypes = len(type_map)

    conf_path = os.path.abspath(conf_dir)
    task_path = re.sub('confs', global_task_name, conf_path)
    if 'relax_incar' in jdata.keys():
        vasp_str='vasp-relax_incar'
        lmps_str= task_type + '-reprod-relax_incar'
    else:
        kspacing = jdata['vasp_params']['kspacing']
        vasp_str = 'vasp-k%.2f' % (kspacing)
        lmps_str = task_type + '-reprod-k%.2f' % (kspacing)

    vasp_path = os.path.join(task_path, vasp_str)
    lmps_path = os.path.join(task_path, lmps_str)

    os.makedirs(lmps_path, exist_ok = True)
    copy_str = "%sx%sx%s" % (supercell[0], supercell[1], supercell[2])
    struct_widecard = os.path.join(vasp_path, 'struct-%s-%s-*' % (insert_ele,copy_str))
    vasp_struct = glob.glob(struct_widecard)
    assert len(vasp_struct)>0 ,"Please compute the interstitial defect using vasp first"
    vasp_struct.sort()
    cwd=os.getcwd()

    # make lammps.in
    if task_type =='deepmd':
        fc = lammps.make_lammps_eval('conf.lmp',
                                 ntypes,
                                 lammps.inter_deepmd,
                                 model_param)
    elif task_type =='meam':
        fc = lammps.make_lammps_eval('conf.lmp',
                                 ntypes,
                                 lammps.inter_meam,
                                 model_param)
    f_lammps_in = os.path.join(lmps_path, 'lammps.in')
    with open(f_lammps_in, 'w') as fp :
        fp.write(fc)

    for vs in vasp_struct :
        # get vasp energy
        outcar = os.path.join(vs, 'OUTCAR')
        energies = vasp.get_energies(outcar)
        # get xdat
        xdatcar = os.path.join(vs, 'XDATCAR')
        struct_basename  = os.path.basename(vs)
        ls = os.path.join(lmps_path, struct_basename)
        print(ls)
        os.makedirs(ls, exist_ok = True)
        os.chdir(ls)
        if os.path.exists('XDATCAR') :
            os.remove('XDATCAR')
        os.symlink(os.path.relpath(xdatcar), 'XDATCAR')
        xdat_lines = open('XDATCAR', 'r').read().split('\n')
        natoms = vasp.poscar_natoms('XDATCAR')
        xdat_secsize = natoms + 8
        xdat_nframes = len(xdat_lines) // xdat_secsize
        if xdat_nframes > len(energies) :
            warnings.warn('nframes %d in xdat is larger than energy %d, use the last %d frames' % (xdat_nframes, len(energies), len(energies)))
            xdat_nlines = len(energies) * xdat_secsize
            xdat_lines = xdat_lines[xdat_nlines:]
        xdat_nframes = len(xdat_lines) // xdat_secsize
        print(xdat_nframes, len(energies))
        #link lammps.in and model
        for jj in ['lammps.in'] + model_name :
            if os.path.islink(jj):
                os.unlink(jj)
        os.symlink(os.path.relpath(f_lammps_in), 'lammps.in')

        for ii in model_name :
            if os.path.exists(ii) :
                os.remove(ii)
        for (ii,jj) in zip(models, model_name) :
            os.symlink(os.path.relpath(ii), jj)
        share_models = [os.path.join(ls,ii) for ii in model_name]

        # loop over frames
        for ii in range(xdat_nframes) :
            frame_path = 'frame.%06d' % ii
            os.makedirs(frame_path, exist_ok=True)
            os.chdir(frame_path)
            # clear dir
            for jj in ['conf.lmp'] :
                if os.path.isfile(jj):
                    os.remove(jj)
            for jj in ['lammps.in'] + model_name :
                if os.path.islink(jj):
                    os.unlink(jj)
            # link lammps in
            os.symlink(os.path.relpath('../lammps.in'), 'lammps.in')
            # make conf
            with open('POSCAR', 'w') as fp :
                fp.write('\n'.join(xdat_lines[ii*xdat_secsize:(ii+1)*xdat_secsize]))
            lammps.cvt_lammps_conf('POSCAR', 'conf.lmp')
            ptypes = vasp.get_poscar_types('POSCAR')
            lammps.apply_type_map('conf.lmp', type_map, ptypes)
            # link models
            for (kk,ll) in zip(share_models, model_name) :
                os.symlink(os.path.relpath(kk), ll)
            os.chdir(ls)
        os.chdir(cwd)



def make_lammps(jdata, conf_dir, supercell, insert_ele, task_type) :
    for ii in insert_ele:
        _make_lammps(jdata, conf_dir, supercell, ii, task_type)

def _make_lammps(jdata, conf_dir, supercell, insert_ele, task_type) :

    fp_params = jdata['lammps_params']
    model_dir = fp_params['model_dir']
    type_map = fp_params['type_map']
    model_dir = os.path.abspath(model_dir)
    model_name =fp_params['model_name']
    deepmd_version = fp_params.get("deepmd_version", "0.12")
    if not model_name and task_type=='deepmd':
        models = glob.glob(os.path.join(model_dir, '*pb'))
        model_name = [os.path.basename(ii) for ii in models]
        assert len(model_name)>0,"No deepmd model in the model_dir"
    else:
        models = [os.path.join(model_dir,ii) for ii in model_name]

    model_param = {'model_name' :      model_name,
                  'param_type':          fp_params['model_param_type'],
                  'deepmd_version' : deepmd_version}

    ntypes = len(type_map)

    conf_path = os.path.abspath(conf_dir)
    conf_poscar = os.path.join(conf_path, 'POSCAR')
    # get equi poscar
    equi_path = re.sub('confs', global_equi_name, conf_path)
    if 'relax_incar' in jdata.keys():
        vasp_str='vasp-relax_incar'
    else:
        kspacing = jdata['vasp_params']['kspacing']
        vasp_str='vasp-k%.2f' % (kspacing)
    equi_path = os.path.join(equi_path, vasp_str)
    equi_contcar = os.path.join(equi_path, 'CONTCAR')
    #equi_path = os.path.join(equi_path, task_type)
    #equi_dump = os.path.join(equi_path, 'dump.relax')
    assert os.path.exists(equi_contcar),"Please compute the equilibrium state using vasp first"
    task_path = re.sub('confs', global_task_name, conf_path)
    task_path = os.path.join(task_path, task_type)
    os.makedirs(task_path, exist_ok=True)
    task_poscar = os.path.join(task_path, 'POSCAR')
    cwd = os.getcwd()
    os.chdir(task_path)
    if os.path.isfile('POSCAR') :
        os.remove('POSCAR')
    os.symlink(os.path.relpath(equi_contcar), 'POSCAR')
    #lammps.poscar_from_last_dump(equi_dump, task_poscar, type_map)
    os.chdir(cwd)
    # gen structure from equi poscar
    print("task poscar: ", task_poscar)
    ss = Structure.from_file(task_poscar)
    # gen defects
    vds = InterstitialGenerator(ss, insert_ele)
    dss = []
    for jj in vds :
        dss.append(jj.generate_defect_structure(supercell))
    # gen tasks
    cwd = os.getcwd()
    # make lammps.in, relax at 0 bar (scale = 1)
    if task_type=='deepmd':
        fc = lammps.make_lammps_press_relax('conf.lmp',
                                        ntypes,
                                        1,
                                        lammps.inter_deepmd,
                                        model_param)
    elif task_type =='meam':
        fc = lammps.make_lammps_press_relax('conf.lmp',
                                        ntypes,
                                        1,
                                        lammps.inter_meam,
                                        model_param)
    f_lammps_in = os.path.join(task_path, 'lammps.in')
    with open(f_lammps_in, 'w') as fp :
        fp.write(fc)
    # gen tasks
    copy_str = "%sx%sx%s" % (supercell[0], supercell[1], supercell[2])
    cwd = os.getcwd()

    os.chdir(task_path)
    for ii in model_name :
        if os.path.exists(ii) :
            os.remove(ii)
    for (ii,jj) in zip(models, model_name) :
        os.symlink(os.path.relpath(ii), jj)
    share_models = [os.path.join(task_path,ii) for ii in model_name]

    for ii in range(len(dss)) :
        struct_path = os.path.join(task_path, 'struct-%s-%s-%03d' % (insert_ele,copy_str,ii))
        print('# generate %s' % (struct_path))
        os.makedirs(struct_path, exist_ok=True)
        os.chdir(struct_path)
        for jj in ['conf.lmp', 'lammps.in'] + model_name :
            if os.path.isfile(jj):
                os.remove(jj)
        # make conf
        dss[ii].to('POSCAR', 'POSCAR')
        lammps.cvt_lammps_conf('POSCAR', 'conf.lmp')
        ptypes = vasp.get_poscar_types('POSCAR')
        lammps.apply_type_map('conf.lmp', type_map, ptypes)
        # link lammps.in
        os.symlink(os.path.relpath(f_lammps_in), 'lammps.in')
        # link models
        for (ii,jj) in zip(share_models, model_name) :
            os.symlink(os.path.relpath(ii), jj)
        # save supercell
        np.savetxt('supercell.out', supercell, fmt='%d')
    os.chdir(cwd)


def _main() :
    parser = argparse.ArgumentParser(
        description="gen 04.interstitial")
    parser.add_argument('TASK', type=str,
                        help='the task of generation, vasp or lammps')
    parser.add_argument('PARAM', type=str,
                        help='json parameter file')
    parser.add_argument('CONF', type=str,
                        help='the path to conf')
    parser.add_argument('COPY', type=int, nargs = 3,
                        help='define the supercell')
    parser.add_argument('ELEMENT', type=str, nargs = '+',
                        help='the inserted element')
    args = parser.parse_args()

    with open (args.PARAM, 'r') as fp :
        jdata = json.load (fp)

#    print('# generate %s task with conf %s' % (args.TASK, args.CONF))
    if args.TASK == 'vasp':
        make_vasp(jdata, args.CONF, args.COPY, args.ELEMENT)
    elif args.TASK == 'deepmd' or args.TASK=='meam':
        make_lammps(jdata, args.CONF, args.COPY, args.ELEMENT, args.TASK)
    elif args.TASK == 'deepmd-reprod' or args.TASK == 'meam-reprod':
        args.TASK=args.TASK.replace('-reprod','')
        make_reprod_traj(jdata, args.CONF, args.COPY, args.ELEMENT, args.TASK)
    else :
        raise RuntimeError("unknow task ", args.TASK)

if __name__ == '__main__' :
    _main()



import os,sys,sys
import unittest

sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), '..')))
__package__ = 'generator'
from .context import make_dispatcher

class TestDispatcher(unittest.TestCase):
    # def test_ssh_slurm(self):
    #     dis = make_dispatcher({
    #         'batch':    'slurm',
    #         'hostname': 'localhost',
    #         'username': 'wanghan',
    #         'port':     22,
    #         'work_path': '.',
    #     })
    #     self.assertEqual(dis.context.__name__, 'SSHContext')
    #     self.assertEqual(dis.batch.__name__, 'Slurm')

    def test_local_slurm(self):
        dis = make_dispatcher({
            'batch':    'slurm',
            'work_path': '.',
        })
        self.assertEqual(dis.context.__name__, 'LocalContext')
        self.assertEqual(dis.batch.__name__, 'Slurm')

    def test_lazy_local_slurm(self):
        dis = make_dispatcher({
            'batch':    'slurm',
            'lazy_local': True,
            'work_path': '.',
        })
        self.assertEqual(dis.context.__name__, 'LazyLocalContext')
        self.assertEqual(dis.batch.__name__, 'Slurm')

    def test_dep_lazy_local_slurm(self):
        dis = make_dispatcher({
            'machine_type':    'slurm',
            'lazy_local': True,
            'work_path': '.',
        })
        self.assertEqual(dis.context.__name__, 'LazyLocalContext')
        self.assertEqual(dis.batch.__name__, 'Slurm')



import os,sys,json,glob,shutil
import dpdata
import numpy as np
import unittest

sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), '..')))
__package__ = 'generator'
from .context import post_fp
from .context import post_fp_pwscf
from .context import post_fp_siesta
from .context import post_fp_vasp
from .context import post_fp_gaussian
from .context import post_fp_cp2k
from .context import param_file
from .context import param_old_file
from .context import param_pwscf_file
from .context import param_pwscf_old_file
from .context import param_siesta_file
from .context import param_gaussian_file
from .context import param_cp2k_file
from .context import machine_file
from .context import setUpModule
from .comp_sys import test_atom_names
from .comp_sys import test_atom_types
from .comp_sys import test_coord
from .comp_sys import test_cell
from .comp_sys import CompLabeledSys
from .context import param_pwmat_file


class TestPostFPVasp(unittest.TestCase):
    def setUp(self):
        assert os.path.isdir('out_data_post_fp_vasp'), 'out data for post fp vasp should exist'
        if os.path.isdir('iter.000000') :
            shutil.rmtree('iter.000000')
        shutil.copytree('out_data_post_fp_vasp', 'iter.000000')
        self.ref_coord = [[[0, 0, 0], [2.3, 2.3, 2.3]],
                          [[0, 0, 0], [2.2, 2.3, 2.4]]]
        self.ref_cell = [4.6 * np.eye(3), 4.6 * np.eye(3)]
        # type_map = ["Mg", "Al"], Al OUTCAR provided
        self.ref_at = [1, 1]
        self.ref_e = [-1.90811235, -1.89718546]
        self.ref_f = [[[ 0.      ,  0.      ,  0.      ], \
                       [-0.      , -0.      , -0.      ]],\
                      [[-0.110216,  0.      ,  0.110216], \
                       [ 0.110216, -0.      , -0.110216]]]
        self.ref_v = [[[ 1.50816698,  0.        , -0.        ], \
                       [ 0.        ,  1.50816698,  0.        ], \
                       [-0.        ,  0.        ,  1.50816795]],\
                      [[ 1.45208913,  0.        ,  0.03036584], \
                       [ 0.        ,  1.67640928,  0.        ], \
                       [ 0.03036584,  0.        ,  1.45208913]]]
        self.ref_coord = np.array(self.ref_coord)
        self.ref_cell = np.array(self.ref_cell)
        self.ref_at = np.array(self.ref_at, dtype = int)
        self.ref_e = np.array(self.ref_e)
        self.ref_f = np.array(self.ref_f)
        self.ref_v = np.array(self.ref_v)

    def tearDown(self):
        shutil.rmtree('iter.000000')

    def test_post_fp_vasp_0(self):

        with open (param_file, 'r') as fp :
            jdata = json.load (fp)
        jdata['use_ele_temp'] = 2
        post_fp_vasp(0, jdata, rfailed=0.3)

        sys = dpdata.LabeledSystem('iter.000000/02.fp/data.000/', fmt = 'deepmd/raw')
        self.assertEqual(sys.get_nframes(), 2)

        if sys.data['coords'][0][1][0] < sys.data['coords'][1][1][0]:
            idx = [1, 0]
        else :
            idx = [0, 1]
        ref_coord = self.ref_coord[idx]
        ref_cell = self.ref_cell[idx]
        ref_e = self.ref_e[idx]
        ref_f = self.ref_f[idx]
        ref_v = self.ref_v[idx]
        ref_at = self.ref_at

        for ff in range(2) :
            self.assertAlmostEqual(ref_e[ff], sys.data['energies'][ff])
        for ii in range(2) :
            self.assertEqual(ref_at[ff], sys.data['atom_types'][ff])
        for ff in range(2) :
            for ii in range(2) :
                for dd in range(3) :
                    self.assertAlmostEqual(ref_coord[ff][ii][dd],
                                           sys.data['coords'][ff][ii][dd])
                    self.assertAlmostEqual(ref_f[ff][ii][dd],
                                           sys.data['forces'][ff][ii][dd])
        for ff in range(2):
            for ii in range(3) :
                for jj in range(3) :
                    self.assertAlmostEqual(ref_v[ff][ii][jj],
                                           sys.data['virials'][ff][ii][jj], places = 5)
                    self.assertAlmostEqual(ref_cell[ff][ii][jj],
                                           sys.data['cells'][ff][ii][jj])

        self.assertTrue(os.path.isfile('iter.000000/02.fp/data.000/set.000/aparam.npy'))
        aparam = np.load('iter.000000/02.fp/data.000/set.000/aparam.npy')
        natoms = sys.get_natoms()
        self.assertEqual(natoms, 2)
        self.assertEqual(list(list(aparam)[0]), [0,0])
        self.assertEqual(list(list(aparam)[1]), [1,1])


    def test_post_fp_vasp_1(self):

        with open (param_file, 'r') as fp :
            jdata = json.load (fp)
        jdata['use_ele_temp'] = 1
        post_fp_vasp(0, jdata, rfailed=0.3)

        sys = dpdata.LabeledSystem('iter.000000/02.fp/data.001/', fmt = 'deepmd/raw')
        self.assertEqual(sys.get_nframes(), 1)

        # if sys.data['coords'][0][1][0] < sys.data['coords'][1][1][0]:
        #     idx = [0]
        # else :
        idx = [1]
        ref_coord = self.ref_coord[idx]
        ref_cell = self.ref_cell[idx]
        ref_e = self.ref_e[idx]
        ref_f = self.ref_f[idx]
        ref_v = self.ref_v[idx]
        ref_at = self.ref_at

        for ff in range(1) :
            self.assertAlmostEqual(ref_e[ff], sys.data['energies'][ff])
        for ii in range(2) :
            self.assertEqual(ref_at[ff], sys.data['atom_types'][ff])
        for ff in range(1) :
            for ii in range(2) :
                for dd in range(3) :
                    self.assertAlmostEqual(ref_coord[ff][ii][dd],
                                           sys.data['coords'][ff][ii][dd])
                    self.assertAlmostEqual(ref_f[ff][ii][dd],
                                           sys.data['forces'][ff][ii][dd])
        for ff in range(1):
            for ii in range(3) :
                for jj in range(3) :
                    self.assertAlmostEqual(ref_v[ff][ii][jj],
                                           sys.data['virials'][ff][ii][jj], places = 5)
                    self.assertAlmostEqual(ref_cell[ff][ii][jj],
                                           sys.data['cells'][ff][ii][jj])

        fparam = np.load('iter.000000/02.fp/data.001/set.000/fparam.npy')
        self.assertEqual(fparam.shape[0], 1)
        self.assertEqual(list(fparam), [100000])


    def test_post_fp_vasp_2(self):
        with open (param_file, 'r') as fp :
            jdata = json.load (fp)
        jdata['use_ele_temp'] = 1
        with self.assertRaises(RuntimeError):
            post_fp_vasp(0, jdata)


class TestPostFPPWSCF(unittest.TestCase, CompLabeledSys):
    def setUp(self):
        self.places = 5
        self.e_places = 5
        self.f_places = 5
        self.v_places = 2
        assert os.path.isdir('out_data_post_fp_pwscf'), 'out data for post fp pwscf should exist'
        if os.path.isdir('iter.000000') :
            shutil.rmtree('iter.000000')
        shutil.copytree('out_data_post_fp_pwscf', 'iter.000000')
        with open (param_pwscf_file, 'r') as fp :
            jdata = json.load (fp)
        post_fp(0, jdata)
        self.system_1 = dpdata.LabeledSystem('iter.000000/orig', fmt = 'deepmd/raw')
        self.system_2 = dpdata.LabeledSystem('iter.000000/02.fp/data.000', fmt = 'deepmd/raw')

class TestPostFPSIESTA(unittest.TestCase, CompLabeledSys):
    def setUp(self):
        self.places = 5
        self.e_places = 5
        self.f_places = 5
        self.v_places = 5
        assert os.path.isdir('out_data_post_fp_siesta'), 'out data for post fp siesta should exist'
        if os.path.isdir('iter.000000') :
            shutil.rmtree('iter.000000')
        shutil.copytree('out_data_post_fp_siesta', 'iter.000000')
        with open (param_siesta_file, 'r') as fp :
            jdata = json.load (fp)
        post_fp(0, jdata)
        self.system_1 = dpdata.LabeledSystem('iter.000000/orig', fmt = 'deepmd/raw')
        self.system_2 = dpdata.LabeledSystem('iter.000000/02.fp/data.000', fmt = 'deepmd/raw')

class TestPostGaussian(unittest.TestCase, CompLabeledSys):
    def setUp(self):
        self.places = 5
        self.e_places = 5
        self.f_places = 5
        self.v_places = 5
        assert os.path.isdir('out_data_post_fp_gaussian'), 'out data for post fp gaussian should exist'
        if os.path.isdir('iter.000000') :
            shutil.rmtree('iter.000000')
        shutil.copytree('out_data_post_fp_gaussian', 'iter.000000')
        with open (param_gaussian_file, 'r') as fp :
            jdata = json.load (fp)
        post_fp(0, jdata)
        self.system_1 = dpdata.LabeledSystem('iter.000000/orig', fmt = 'deepmd/raw')
        self.system_2 = dpdata.LabeledSystem('iter.000000/02.fp/data.000', fmt = 'deepmd/raw')

class TestPostCP2K(unittest.TestCase, CompLabeledSys):
    def setUp(self):
        self.places = 5
        self.e_places = 5
        self.f_places = 5
        self.v_places = 5
        assert os.path.isdir('out_data_post_fp_cp2k'), 'out data for post fp gaussian should exist'
        if os.path.isdir('iter.000000') :
            shutil.rmtree('iter.000000')
        shutil.copytree('out_data_post_fp_cp2k', 'iter.000000')
        with open (param_cp2k_file, 'r') as fp :
            jdata = json.load (fp)
        post_fp(0, jdata)
        self.system_1 = dpdata.LabeledSystem('iter.000000/orig', fmt = 'deepmd/raw')
        self.system_2 = dpdata.LabeledSystem('iter.000000/02.fp/data.000', fmt = 'deepmd/raw')


class TestPostFPPWmat(unittest.TestCase, CompLabeledSys):
    def setUp(self):
        self.places = 5
        self.e_places = 5
        self.f_places = 5
        self.v_places = 2
        assert os.path.isdir('out_data_post_fp_pwmat'), 'out data for post fp pwmat should exist'
        if os.path.isdir('iter.000000') :
            shutil.rmtree('iter.000000')
        shutil.copytree('out_data_post_fp_pwmat', 'iter.000000')
        with open (param_pwmat_file, 'r') as fp :
            jdata = json.load (fp)
        post_fp(0, jdata)
        self.system_1 = dpdata.LabeledSystem('iter.000000/orig', fmt = 'deepmd/raw')
        self.system_2 = dpdata.LabeledSystem('iter.000000/02.fp/data.000', fmt = 'deepmd/raw')


if __name__ == '__main__':
    unittest.main()



import sys,os

sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), '..', '..')))
from dpgen.tools.run_report import *

def my_file_cmp(test, f0, f1):
    with open(f0) as fp0 :
        with open(f1) as fp1:
            test.assertTrue(fp0.read() == fp1.read())




#!/usr/bin/env python
# coding: utf-8

import os,sys,glob,time
import numpy as np
import subprocess as sp
from monty.serialization import dumpfn,loadfn
from dpgen.remote.RemoteJob import SlurmJob, PBSJob, CloudMachineJob, JobStatus, awsMachineJob,SSHSession
from dpgen import dlog

import requests
from hashlib import sha1

def _verfy_ac(private_key, params):
    items= sorted(params.items())
    
    params_data = ""
    for key, value in items:
        params_data = params_data + str(key) + str(value)
    params_data = params_data + private_key
    sign = sha1()
    sign.update(params_data.encode())
    signature = sign.hexdigest()
    return signature

def aws_submit_jobs(machine,
                    resources,
                    command,
                    work_path,
                    tasks,
                    group_size,
                    forward_common_files,
                    forward_task_files,
                    backward_task_files, 
                    forward_task_deference = True):
    import boto3
    task_chunks = [
        [os.path.basename(j) for j in tasks[i:i + group_size]] \
        for i in range(0, len(tasks), group_size)
    ]
    task_chunks = (str(task_chunks).translate((str.maketrans('','',' \'"[]'))).split(',')) 
    # flatten the task_chunks
    print('task_chunks=',task_chunks)
    njob = len(task_chunks)    
    print('njob=',njob)
    continue_status = False
    ecs=boto3.client('ecs')
    ec2=boto3.client('ec2')
    status_list=[]    
    containerInstanceArns=ecs.list_container_instances(cluster="tensorflow")    
    if  containerInstanceArns['containerInstanceArns']:
        containerInstances=ecs.describe_container_instances(cluster="tensorflow", \
                                                    containerInstances=containerInstanceArns['containerInstanceArns'])['containerInstances']
        status_list=[container['status'] for container in containerInstances]

    need_apply_num=group_size-len(status_list)
    print('need_apply_num=',need_apply_num)
    if need_apply_num>0:
        for ii in range(need_apply_num) :   #apply for machines,
            ec2.run_instances(**machine['run_instances'])      
    machine_fin = False    
    status_list=[]
    while not len(status_list)>=group_size:
        containerInstanceArns=ecs.list_container_instances(cluster="tensorflow")
        if  containerInstanceArns['containerInstanceArns']:
            containerInstances=ecs.describe_container_instances(cluster="tensorflow", \
                                                        containerInstances=containerInstanceArns['containerInstanceArns'])['containerInstances']
            status_list=[container['status'] for container in containerInstances]            
        if len(status_list)>=group_size:
            break
        else:
            time.sleep(20)
    print('current available containers status_list=',status_list)       
    print('remote_root=',machine['remote_root'])   
    rjob = awsMachineJob(machine['remote_root'],work_path)
    taskARNs=[]
    taskstatus=[]
    running_job_num=0
    rjob.upload('.',  forward_common_files)
    for ijob in range(njob) :   #uplaod && submit job
        containerInstanceArns=ecs.list_container_instances(cluster="tensorflow")
        containerInstances=ecs.describe_container_instances(cluster="tensorflow", \
                                                    containerInstances=containerInstanceArns['containerInstanceArns'])['containerInstances']
        status_list=[container['status'] for container in containerInstances]
        print('current available containers status_list=',status_list)      
        while  running_job_num>=group_size:
            taskstatus=[task['lastStatus'] for task in ecs.describe_tasks(cluster='tensorflow',tasks=taskARNs)['tasks']]
            running_job_num=len(list(filter(lambda str:(str=='PENDING' or str =='RUNNING'),taskstatus)))
            print('waiting for running job finished, taskstatus=',taskstatus,'running_job_num=',running_job_num)
            time.sleep(10)         
        chunk = str(task_chunks[ijob])
        print('current task chunk=',chunk)
        task_definition=command['task_definition']
        concrete_command=(command['concrete_command'] %(work_path,chunk))
        command_override=command['command_override']
        command_override['containerOverrides'][0]['command'][0]=concrete_command
        print('concrete_command=',concrete_command)
        rjob.upload(chunk, forward_task_files, 
                    dereference = forward_task_deference)
        taskres=ecs.run_task(cluster='tensorflow',\
                     taskDefinition=task_definition,overrides=command_override)
        while not taskres['tasks'][0]:
            print('task submit failed,taskres=',taskres,'trying to re-submit'+str(chunk),)
            time.sleep(10)
            taskres=ecs.run_task(cluster='tensorflow',\
                     taskDefinition=task_definition,overrides=command_override)

        taskARNs.append(taskres['tasks'][0]['taskArn'])
        taskstatus=[task['lastStatus'] for task in ecs.describe_tasks(cluster='tensorflow',tasks=taskARNs)['tasks']]
        running_job_num=len(list(filter(lambda str:(str=='PENDING' or str =='RUNNING'),taskstatus)))
        print('have submitted %s/%s,taskstatus=' %(work_path,chunk) ,taskstatus,'running_job_num=',running_job_num )       
    task_fin_flag=False    
    while not task_fin_flag:       
        taskstatus=[task['lastStatus'] for task in ecs.describe_tasks(cluster='tensorflow',tasks=taskARNs)['tasks']]
        task_fin_flag=all([status=='STOPPED' for status in taskstatus])
        if task_fin_flag:
            print('task finished,next step:copy files to local && taskstatus=',taskstatus)
        else:
            print('all tasks submitted,task running && taskstatus=',taskstatus)
            time.sleep(20)    
    for ii in range(njob):
        chunk = task_chunks[ii]
        print('downloading '+str(chunk),backward_task_files)
        rjob.download(chunk,backward_task_files)

def _ucloud_remove_machine(machine, UHostId):
    ucloud_url = machine['url']
    ucloud_stop_param = {}
    ucloud_stop_param['Action'] = "StopUHostInstance"
    ucloud_stop_param['Region'] = machine['ucloud_param']['Region']
    ucloud_stop_param['UHostId'] = UHostId
    ucloud_stop_param['PublicKey'] = machine['ucloud_param']['PublicKey']
    ucloud_stop_param['Signature'] = _verfy_ac(machine['Private'], ucloud_stop_param)

    
    req = requests.get(ucloud_url, ucloud_stop_param)
    if req.json()['RetCode'] != 0 :
        raise RuntimeError ("failed to stop ucloud machine")

    terminate_fin = False
    try_time = 0 
    while not terminate_fin:
        ucloud_delete_param = {}
        ucloud_delete_param['Action'] = "TerminateUHostInstance"
        ucloud_delete_param['Region'] = machine['ucloud_param']['Region']
        ucloud_delete_param['UHostId'] = UHostId
        ucloud_delete_param['PublicKey'] = machine['ucloud_param']['PublicKey']
        ucloud_delete_param['Signature'] = _verfy_ac(machine['Private'], ucloud_delete_param)    
        req = requests.get(ucloud_url, ucloud_delete_param)
        if req.json()['RetCode'] == 0 :
            terminate_fin = True
        try_time = try_time + 1
        if try_time >= 200:
            raise RuntimeError ("failed to terminate ucloud machine")
        time.sleep(10)
    print("Machine ",UHostId,"has been successfully terminated!")   

def ucloud_submit_jobs(machine,
                       resources,
                       command,
                       work_path,
                       tasks,
                       group_size,
                       forward_common_files,
                       forward_task_files,
                       backward_task_files, 
                       forward_task_deference = True) :
    task_chunks = [
        [os.path.basename(j) for j in tasks[i:i + group_size]] \
        for i in range(0, len(tasks), group_size)
    ]
    njob = len(task_chunks)
    continue_status = False
    if os.path.isfile("record.machine"):
        with open ("record.machine", "r") as fr:
            record_machine = json.load(fr)
            if record_machine["purpose"] == machine["purpose"] and record_machine["njob"] == njob:
                continue_status = True
                ucloud_machines = record_machine["ucloud_machines"]
                ucloud_hostids = record_machine["ucloud_hostids"]
        fr.close()
    ucloud_url = machine['url']
    if continue_status == False:
        assert machine['machine_type'] == 'ucloud'
        ucloud_start_param = machine['ucloud_param']
        ucloud_start_param['Action'] = "CreateUHostInstance"
        ucloud_start_param['Name'] = "train"
        ucloud_start_param['Signature'] = _verfy_ac(machine['Private'], ucloud_start_param)

        
        ucloud_machines = []
        ucloud_hostids = []
        for ii in range(njob) :
            req = requests.get(ucloud_url, ucloud_start_param)
            if req.json()['RetCode'] != 0 :
                print(json.dumps(req.json(),indent=2, sort_keys=True))
                raise RuntimeError ("failed to start ucloud machine")
            ucloud_machines.append(str(req.json()["IPs"][0]))
            ucloud_hostids.append(str(req.json()["UHostIds"][0]))

        new_record_machine = {}
        new_record_machine["purpose"] = machine["purpose"]
        new_record_machine["njob"] = njob
        new_record_machine["ucloud_machines"] = ucloud_machines
        new_record_machine["ucloud_hostids"] = ucloud_hostids
        with open ("record.machine", "w") as fw:
            json.dump(new_record_machine, fw)
        fw.close()

    machine_fin = [False for ii in ucloud_machines]
    total_machine_num = len(ucloud_machines)
    fin_machine_num = 0
    while not all(machine_fin):
        for idx,mac in enumerate(ucloud_machines):
            if not machine_fin[idx]:
                ucloud_check_param = {}
                ucloud_check_param['Action'] = "GetUHostInstanceVncInfo"
                ucloud_check_param['Region'] = machine['ucloud_param']['Region']
                ucloud_check_param['UHostId'] = ucloud_hostids[idx]
                ucloud_check_param['PublicKey'] = machine['ucloud_param']['PublicKey']
                ucloud_check_param['Signature'] = _verfy_ac(machine['Private'], ucloud_check_param)
                req = requests.get(ucloud_url, ucloud_check_param)
                print("the UHostId is", ucloud_hostids[idx])
                print(json.dumps(req.json(),indent=2, sort_keys=True))
                if req.json()['RetCode'] == 0 :
                    machine_fin[idx] = True
                    fin_machine_num = fin_machine_num + 1
        print("Current finish",fin_machine_num,"/", total_machine_num)

        
        ucloud_check_param1 = {}
        ucloud_check_param1['Action'] = "DescribeUHostInstance"
        ucloud_check_param1['Region'] = machine['ucloud_param']['Region']
        ucloud_check_param1["Limit"] = 100
        ucloud_check_param1['PublicKey'] = machine['ucloud_param']['PublicKey']
        ucloud_check_param1['Signature'] = _verfy_ac(machine['Private'], ucloud_check_param1)
        req1 = requests.get(ucloud_url, ucloud_check_param1).json()
        
        machine_all_fin = True
        for idx1 in range(int(req1["TotalCount"])):
            if req1["UHostSet"][idx1]["State"] != "Running":
                machine_all_fin = False
                break
        if machine_all_fin == True:
            machine_fin = [True for i in machine_fin]
        time.sleep(10)
    ssh_sess = []
    ssh_param = {}
    ssh_param['port'] = 22
    ssh_param['username'] = 'root'
    ssh_param['work_path'] = machine['work_path']
    for ii in ucloud_machines :
        ssh_param['hostname'] = ii
        ssh_sess.append(SSHSession(ssh_param))

    job_list = []
    for ii in range(njob) :
        chunk = task_chunks[ii]
        print("Current machine is", ucloud_machines[ii])
        rjob = CloudMachineJob(ssh_sess[ii], work_path)
        rjob.upload('.',  forward_common_files)
        rjob.upload(chunk, forward_task_files, 
                    dereference = forward_task_deference)
        rjob.submit(chunk, command, resources = resources)
        job_list.append(rjob)
    
    job_fin = [False for ii in job_list]
    while not all(job_fin) :
        for idx,rjob in enumerate(job_list) :
            if not job_fin[idx] :
                status = rjob.check_status()
                if status == JobStatus.terminated :
                    raise RuntimeError("find unsuccessfully terminated job on machine" % ucloud_machines[idx])
                elif status == JobStatus.finished :
                    rjob.download(task_chunks[idx], backward_task_files)
                    rjob.clean()
                    _ucloud_remove_machine(machine, ucloud_hostids[idx])
                    job_fin[idx] = True
        time.sleep(10)
    os.remove("record.machine")


def group_slurm_jobs(ssh_sess,
                     resources,
                     command,
                     work_path,
                     tasks,
                     group_size,
                     forward_common_files,
                     forward_task_files,
                     backward_task_files,
                     remote_job = SlurmJob, 
                     forward_task_deference = True) :

    task_chunks = [
        [os.path.basename(j) for j in tasks[i:i + group_size]] \
        for i in range(0, len(tasks), group_size)
    ]
    cwd=os.getcwd()
    _pmap=PMap(cwd)
    path_map=_pmap.load()
    dlog.debug("work_path: %s"% work_path)
    dlog.debug("curr_path: %s"% cwd)

    job_list = []
    task_chunks_=['+'.join(ii) for ii in task_chunks]
    for ii in task_chunks_:
        dlog.debug("task_chunk %s" % ii)

    #dlog.debug(path_map)
    for ii,chunk in enumerate(task_chunks) :
       
        # map chunk info. to uniq id    
        chunk_uni=task_chunks_[ii].encode('utf-8')
        chunk_sha1=sha1(chunk_uni).hexdigest() 

        if chunk_sha1 in path_map:
           job_uuid=path_map[chunk_sha1][1].split('/')[-1]
           dlog.debug("load uuid %s" % job_uuid)
        else:
           job_uuid=None

        rjob = remote_job(ssh_sess, work_path, job_uuid)
        dlog.debug('uuid %s'%job_uuid)
        rjob.upload('.',  forward_common_files)
        rjob.upload(chunk, forward_task_files, 
                   dereference = forward_task_deference)
        if job_uuid:
           rjob.submit(chunk, command, resources = resources,restart=True)
        else:
           rjob.submit(chunk, command, resources = resources)
        job_list.append(rjob)
        path_map[chunk_sha1]=[rjob.local_root,rjob.remote_root]        
    _pmap.dump(path_map)
    
    job_fin = [False for ii in job_list]
    lcount=[0]*len(job_list)
    count_fail = 0
    while not all(job_fin) :
        for idx,rjob in enumerate(job_list) :
            if not job_fin[idx] :
                try:
                  status = rjob.check_status()
                except:
                  ssh_sess = SSHSession(ssh_sess.remote_profile)
                  for _idx,_rjob in enumerate(job_list):
                    job_list[_idx] = SlurmJob(ssh_sess, work_path, _rjob.job_uuid)
                  count_fail = count_fail +1
                  dlog.info("ssh_sess failed for %d times"%count_fail)
                  break
                if status == JobStatus.terminated :
                    lcount[idx]+=1
                    _job_uuid=rjob.remote_root.split('/')[-1]
                    dlog.info('Job at %s  terminated, submit again'% _job_uuid)
                    dlog.debug('try %s times for %s'% (lcount[idx], _job_uuid))
                    rjob.submit(task_chunks[idx], command, resources = resources,restart=True)
                    if lcount[idx]>3:
                       dlog.info('Too many errors for ! %s ' % _job_uuid)
                       rjob.download(task_chunks[idx], backward_task_files,back_error=True)
                       rjob.clean()
                       job_fin[idx] = True
                elif status == JobStatus.finished :
                    rjob.download(task_chunks[idx], backward_task_files)
                    rjob.clean()
                    job_fin[idx] = True
        time.sleep(10)
    dlog.debug('error count') 
    dlog.debug(lcount)
    # delete path map file when job finish
    _pmap.delete()

def group_local_jobs(ssh_sess,
                     resources,
                     command,
                     work_path,
                     tasks,
                     group_size,
                     forward_common_files,
                     forward_task_files,
                     backward_task_files, 
                     forward_task_deference = True) :
    task_chunks = [
        [os.path.basename(j) for j in tasks[i:i + group_size]] \
        for i in range(0, len(tasks), group_size)
    ]
    job_list = []
    for chunk in task_chunks :
        rjob = CloudMachineJob(ssh_sess, work_path)
        rjob.upload('.',  forward_common_files)
        rjob.upload(chunk, forward_task_files, 
                    dereference = forward_task_deference)
        rjob.submit(chunk, command, resources = resources)
        job_list.append(rjob)
        job_fin = False
        while not job_fin :
            status = rjob.check_status()
            if status == JobStatus.terminated :
                raise RuntimeError("find unsuccessfully terminated job in %s" % rjob.get_job_root())
            elif status == JobStatus.finished :
                rjob.download(chunk, backward_task_files)
                rjob.clean()
                job_fin = True
            time.sleep(10)

class PMap(object):
   '''
   Path map class to operate {read,write,delte} the pmap.json file
   '''

   def __init__(self,path,fname="pmap.json"):
       self.f_path_map=os.path.join(path,fname)

   def load(self):
      f_path_map=self.f_path_map
      if os.path.isfile(f_path_map):
         path_map=loadfn(f_path_map)
      else:
         path_map={}
      return path_map

   def dump(self,pmap,indent=4):
      f_path_map=self.f_path_map
      dumpfn(pmap,f_path_map,indent=indent)

   def delete(self):
      f_path_map=self.f_path_map
      try:
         os.remove(f_path_map)
      except:
         pass



#!/usr/bin/env python3

import os, re, argparse, filecmp, json, glob, sys
import subprocess as sp
import numpy as np
import dpgen.auto_test.lib.vasp as vasp
import dpgen.auto_test.lib.lammps as lammps
from pymatgen.analysis.elasticity.elastic import ElasticTensor
from pymatgen.analysis.elasticity.strain import Strain
from pymatgen.analysis.elasticity.stress import Stress

global_equi_name = '00.equi'
global_task_name = '03.vacancy'

def comput_e_shift(poscar, task_name) :
    a_types = vasp.get_poscar_types(poscar)
    a_natoms = vasp.get_poscar_natoms(poscar)
    ener_shift = 0
    if not os.path.isdir('stables') :
        raise RuntimeError('no dir "stable". Stable energy and volume of components should be computed before calculating formation energy of an alloy')
    for ii in range(len(a_types)) :
        ref_e_file = a_types[ii] + '.' + task_name + '.e'
        ref_e_file = os.path.join('stables', ref_e_file)
        ener = float(open(ref_e_file).read())
        ener_shift += a_natoms[ii] * ener
    return ener_shift

def cmpt_vasp(jdata, conf_dir, supercell) :

    if 'relax_incar' in jdata.keys():
        vasp_str='vasp-relax_incar'
    else:
        kspacing = jdata['vasp_params']['kspacing']
        vasp_str='vasp-k%.2f' % kspacing

    equi_path = re.sub('confs', global_equi_name, conf_dir)
    equi_path = os.path.join(equi_path, vasp_str)
    equi_path = os.path.abspath(equi_path)
    equi_outcar = os.path.join(equi_path, 'OUTCAR')
    task_path = re.sub('confs', global_task_name, conf_dir)
    task_path = os.path.join(task_path, vasp_str)
    task_path = os.path.abspath(task_path)
    print("# ", task_path)

    equi_natoms, equi_epa, equi_vpa = vasp.get_nev(equi_outcar)

    copy_str = "%sx%sx%s" % (supercell[0], supercell[1], supercell[2])
    struct_path_widecard = os.path.join(task_path, 'struct-%s-*' % (copy_str))
    struct_path_list = glob.glob(struct_path_widecard)
    struct_path_list.sort()
    if len(struct_path_list) == 0:
        print("# cannot find results for conf %s supercell %s" % (conf_dir, supercell))
    sys.stdout.write ("Structure: \tVac_E(eV)  E(eV) equi_E(eV)\n")
    result = os.path.join(task_path,'result')
    with open(result,'w') as fp:
        fp.write('conf_dir:%s\n'% (conf_dir))
        fp.write("Structure: \tVac_E(eV)  E(eV) equi_E(eV)\n")
        for ii in struct_path_list :
            struct_poscar = os.path.join(ii, 'POSCAR')
            #energy_shift = comput_e_shift(struct_poscar, vasp_str)
            structure_dir = os.path.basename(ii)
            outcar = os.path.join(ii, 'OUTCAR')
            natoms, epa, vpa = vasp.get_nev(outcar)
            evac = epa * natoms - equi_epa * natoms
            sys.stdout.write ("%s: %7.3f  %7.3f %7.3f \n" % (structure_dir, evac, epa * natoms, equi_epa*natoms))
            fp.write("%s: %7.3f  %7.3f %7.3f \n" % (structure_dir, evac, epa * natoms, equi_epa*natoms))
    fp.close()
        # evac = epa * natoms - energy_shift
        # sys.stdout.write ("%s: %7.3f  %7.3f %7.3f \n" % (structure_dir, evac, epa * natoms, energy_shift))
        # sys.stdout.write ("%s: %7.3f \n" % (structure_dir, evac))

def cmpt_deepmd_lammps(jdata, conf_dir, supercell, task_name) :
    equi_path = re.sub('confs', global_equi_name, conf_dir)
    equi_path = os.path.join(equi_path, task_name)
    equi_path = os.path.abspath(equi_path)
    equi_log = os.path.join(equi_path, 'log.lammps')
    task_path = re.sub('confs', global_task_name, conf_dir)
    task_path = os.path.join(task_path, task_name)
    task_path = os.path.abspath(task_path)
    print("# ", task_path)

    equi_natoms, equi_epa, equi_vpa = lammps.get_nev(equi_log)

    copy_str = "%sx%sx%s" % (supercell[0], supercell[1], supercell[2])
    struct_path_widecard = os.path.join(task_path, 'struct-%s-*' % (copy_str))
    struct_path_list = glob.glob(struct_path_widecard)
    struct_path_list.sort()
    if len(struct_path_list) == 0:
        print("# cannot find results for conf %s supercell %s" % (conf_dir, supercell))
    sys.stdout.write ("Structure: \tVac_E(eV)  E(eV) equi_E(eV)\n")
    result = os.path.join(task_path,'result')
    with open(result,'w') as fp:
        fp.write('conf_dir:%s\n'% (conf_dir))
        fp.write("Structure: \tVac_E(eV)  E(eV) equi_E(eV)\n")
        for ii in struct_path_list :
            struct_poscar = os.path.join(ii, 'POSCAR')
            #energy_shift = comput_e_shift(struct_poscar, task_name)
            structure_dir = os.path.basename(ii)
            lmp_log = os.path.join(ii, 'log.lammps')
            natoms, epa, vpa = lammps.get_nev(lmp_log)
            evac = epa * natoms - equi_epa * natoms
            sys.stdout.write ("%s: %7.3f  %7.3f %7.3f \n" % (structure_dir, evac, epa * natoms, equi_epa * natoms))
            fp.write("%s: %7.3f  %7.3f %7.3f \n" % (structure_dir, evac, epa * natoms, equi_epa*natoms))
    fp.close()
        # evac = epa * natoms - energy_shift
        # sys.stdout.write ("%s: %7.3f  %7.3f %7.3f \n" % (structure_dir, evac, epa * natoms, energy_shift))
        # sys.stdout.write ("%s: %7.3f\n" % (structure_dir, evac))

def _main() :
    parser = argparse.ArgumentParser(
        description="cmpt 03.vacancy")
    parser.add_argument('TASK', type=str,
                        help='the task of generation, vasp or lammps')
    parser.add_argument('PARAM', type=str,
                        help='json parameter file')
    parser.add_argument('CONF', type=str,
                        help='the path to conf')
    parser.add_argument('COPY', type=int, nargs = 3,
                        help='the path to conf')
    args = parser.parse_args()

    with open (args.PARAM, 'r') as fp :
        jdata = json.load (fp)

#    print('# generate %s task with conf %s' % (args.TASK, args.CONF))
    if args.TASK == 'vasp':
        cmpt_vasp(jdata, args.CONF, args.COPY)
    elif args.TASK == 'deepmd' :
        cmpt_deepmd_lammps(jdata, args.CONF, args.COPY, args.TASK)
    elif args.TASK == 'meam' :
        cmpt_deepmd_lammps(jdata, args.CONF, args.COPY, args.TASK)
    else :
        raise RuntimeError("unknow task ", args.TASK)

if __name__ == '__main__' :
    _main()



#!/usr/bin/env python2
################################################################
#
# vasp2force:
#   convert vasp output data into potfit reference configurations
#
################################################################
#
#   Copyright 2014
#             Institute for Theoretical and Applied Physics
#             University of Stuttgart, D-70550 Stuttgart, Germany
#             http://potfit.sourceforge.net/
#
#################################################################
#
#   This file is part of potfit.
#
#   potfit is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   potfit is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with potfit; if not, see <http://www.gnu.org/licenses/>.
#
#################################################################

import argparse
import copy
import gzip
import os
import sys


def get_outcar_files(directory, recursive):
    # walk directory (recursively) and return all OUTCAR* files
    # return list of outcars' path
    sys.stderr.write(
        'Searching directory %s for OUTCAR* files ...\n' % directory)
    outcars = []
    if not recursive:
        for item in os.listdir(directory):
            if item.startswith('OUTCAR'):
                outcars.append(os.path.join(directory, item))
    else:
        for root, SubFolders, files in os.walk(directory):
            for item in files:
                if item.startswith('OUTCAR'):
                    outcars.append(os.path.join(root, item))
    if len(outcars) == 0:
        sys.stderr.write(
            'Could not find any OUTCAR files in this directory.\n')
    else:
        sys.stderr.write('Found the following files:\n')
        sys.stderr.write('  {}\n'.format(('\n  ').join(outcars)))
        return outcars
    return outcars


def uniq(seq):
    # return unique list without changing the order
    # from http://stackoverflow.com/questions/480214
    seen = set()
    seen_add = seen.add
    return [x for x in seq if x not in seen and not seen_add(x)]


def scan_outcar_file(file_handle):
    # scans an outcar file and returns a list with
    # - number of configurations
    # - atom types

    # first try TOTAL-FORCE
    configs = 0
    atom_types = []
    title = []
    potcar = []
    ipt = []
    for line in file_handle:
        if line.startswith('|'):
            continue
        if 'TOTAL-FORCE' in line:
            configs += 1
        if 'VRHFIN' in line:
            atom_types.append(
                line.split()[1].replace('=', '').replace(':', ''))
        if 'title' in line:
            title.append(line.split()[3][0:2])
        if 'POTCAR' in line:
            potcar.append(line.split()[2][0:2])
        if 'ions per type' in line:
            ipt = [int(s) for s in line.split()[4:]]

    potcar = uniq(potcar)

    if atom_types:
        return [configs, atom_types, ipt]
    elif title:
        return [configs, title, ipt]
    elif potcar:
        return [configs, potcar, ipt]
    else:
        sys.stderr.write(
            'Could not determine atom types in file %s.\n' % filename)
        sys.exit()


def process_outcar_file_v5_dev(outcars, data, numbers, types, max_types, elements=None, windex=None, fout='potfit.configs'):
    fw = open(fout, 'w')
    for i in range(len(outcars)):
        if outcars[i].endswith('.gz'):
            f = gzip.open(outcars[i], 'rb')
        else:
            f = open(outcars[i], 'r')

        # Writing current OUTCAR's information into potfit format files.
        nconfs = data[i][0]
        natoms = sum(data[i][2])  # ipt
        if windex == None:
            windex = range(nconfs)
        if windex == 'final':
            windex = [nconfs - 1]

        # reading current OUTCAR
        print("Reading %s ..." % outcars[i])
        count = -1
        line = f.readline()
        while line != '':
            line = f.readline()
            if 'Iteration' in line:
                energy = 0
                box_x = []
                box_y = []
                box_z = []
                stress = []
                atom_data = []
            # if 'energy  without' in line:
            #     # appears in each electronic-iteration steps
            #     energy = float(line.split()[6]) / natoms
            if 'free  energy   TOTEN' in line:
                energy = float (line.split()[4]) / natoms
                if count in windex:
                    fw.write("#N %s 1\n" % natoms)
                    fw.write('#C ')
                    if elements:
                        fw.write("%s " % numbers[0])
                        for j in range(1, max_types):
                            fw.write('%s\t' % numbers[j])
                    else:
                        fw.write(" %s" % data[i][1][0])
                        for j in range(1, max_types):
                            fw.write(' %s' % data[i][1][j])
                    fw.write("\n")
                    fw.write("## force file generated from file %s config %d\n" % (
                        outcars[i], count))
                    fw.write("#X %13.8f %13.8f %13.8f\n" %
                             (box_x[0], box_x[1], box_x[2]))
                    fw.write("#Y %13.8f %13.8f %13.8f\n" %
                             (box_y[0], box_y[1], box_y[2]))
                    fw.write("#Z %13.8f %13.8f %13.8f\n" %
                             (box_z[0], box_z[1], box_z[2]))
                    fw.write("#W %f\n" % (args.weight))
                    fw.write("#E %.10f\n" % (energy))
                    if stress:
                        fw.write("#S ")
                        for num in range(6):
                            fw.write('%8.7g\t' % (stress[num]))
                        fw.write('\n')
                    fw.write("#F\n")
                    fw.flush()
                    for adata in atom_data:
                        fw.write("%d %11.7g %11.7g %11.7g %11.7g %11.7g %11.7g\n" %
                                 (adata[0], adata[1], adata[2], adata[3], adata[4], adata[5], adata[6]))
            if 'VOLUME and BASIS' in line:
                for do in range(5):
                    # SKIP 5 lines
                    line = f.readline()
                box_x = [float(s)
                         for s in line.replace('-', ' -').split()[0:3]]
                line = f.readline()
                box_y = [float(s)
                         for s in line.replace('-', ' -').split()[0:3]]
                line = f.readline()
                box_z = [float(s)
                         for s in line.replace('-', ' -').split()[0:3]]
            if 'in kB' in line:
                stress = [float(s) / 1602 for s in line.split()[2:8]]
            if 'TOTAL-FORCE' in line:
                # only appears in Ion-iteration steps
                line = f.readline()  # skip 1 line
                adata = [0] * 7
                for num in range(len(data[i][2])):
                    for k in range(data[i][2][num]):
                        line = [float(s) for s in f.readline().split()[0:6]]
                        if args.c:
                            adata[0] = int(types[data[i][1][num]])
                        else:
                            adata[0] = int(num)
                        adata[1] = float(line[0])
                        adata[2] = float(line[1])
                        adata[3] = float(line[2])
                        adata[4] = float(line[3])
                        adata[5] = float(line[4])
                        adata[6] = float(line[5])
                        atom_data.append(copy.copy(adata))
                count += 1
    fw.close()
    return


def Parser():
    parser = argparse.ArgumentParser(
        description='''Converts vasp output data into potfit reference configurations.''')

    parser.add_argument('-c', type=str, required=False,
                        help='list of chemical species to use, e.g. -c Mg=0,Zn=1')
    parser.add_argument(
        '-e', type=str, required=False, help='file with single atom energies (NYI)')
    parser.add_argument('-r', '--recursive', action='store_true',
                        help='scan recursively for OUTCAR files')
    parser.add_argument('-f', '--final', action='store_true',
                        help='use only the final configuration from OUTCAR')
    parser.add_argument('-sr', '--configs_range', type=str,
                        help='range of the configurations to use')
    parser.add_argument('-w', '--weight', type=float, default=1.0,
                        help='set configuration weight for all configurations')
    parser.add_argument(
        'files', type=str, nargs='*', help='list of OUTCAR files (plain or gzipped)')

    args = parser.parse_args()
    return args

#
if __name__ == "__main__":
    # Check for sane arguments
    args = Parser()
    if args.weight < 0:
        sys.stderr.write("The weight needs to be positive!\n")
        sys.exit()

    # determine all OUTCAR files
    outcars = []
    if not args.files:
        outcars = get_outcar_files('.', args.recursive)

    for item in args.files:
        if os.path.isdir(item):
            outcars += get_outcar_files(item, args.recursive)
        else:
            outcars.append(os.path.abspath(item))

    # remove duplicate entries from the outcars table
    outcars = uniq(outcars)

    # read metadata from all outcar files
    data = []
    max_types = 1
    for item in outcars:
        if item.endswith('.gz'):
            f = gzip.open(item, 'rb')
        else:
            f = open(item, 'r')
        data.append(scan_outcar_file(f))
        f.close()
        max_types = max(max_types, len(data[-1][1]))

    # check the types string
    types = dict()
    numbers = dict()
    if args.c:
        if len(args.c.split(',')) > max_types:
            sys.stderr.write(
                "\nERROR: There are too many items in you -c string!\n")
            sys.exit()
        if len(args.c.split(',')) < max_types:
            sys.stderr.write(
                "\nERROR: There are not enough items in you -c string!\n")
            sys.exit()
        for item in args.c.split(','):
            if len(item.split('=')) != 2:
                sys.stderr.write("\nERROR: Could not read the -c string.\n")
                sys.stderr.write("Maybe a missing or extra '=' sign?\n")
                sys.exit()
            else:
                try:
                    name = str(item.split('=')[0])
                    number = int(item.split('=')[1])
                except:
                    sys.stderr.write("\nERROR: Could not read the -c string\n")
                    sys.exit()
                if number >= max_types:
                    sys.stderr.write(
                        "\nERROR: The atom type for %s is invalid!\n" % name)
                    sys.exit()
                if name in types:
                    sys.stderr.write(
                        "\nERROR: Duplicate atom type found in -c string\n")
                    sys.exit()
                if number in numbers:
                    sys.stderr.write(
                        "\nERROR: Duplicate atom number found in -c string\n")
                    sys.exit()
                types[name] = number
                numbers[number] = name

    # process all the outcar files
    sr = args.configs_range
    if sr == None:
        windex = None
    else:
        sr = sr.split()
        sr0 = int(sr[0])
        sr1 = int(sr[1])
        windex = range(sr0, sr1 + 1)

    if args.final:
        windex = 'final'

    process_outcar_file_v5_dev(
        outcars, data, numbers, types, max_types, windex=windex, fout='test.configs')



#/usr/bin/env python
# coding: utf-8
# Copyright (c) The Dpmodeling Team.

import json
import warnings
from uuid import uuid4
from dpdata import System,LabeledSystem
from dpgen.database.vasp import VaspInput
from pymatgen.core.composition import Composition
from monty.json import MontyEncoder, MontyDecoder, MSONable

"""
This module implements equivalents of the basic Entry objects, which
is the basic entity that can be used to perform many analyses. Entries
contain calculated information, typically from VASP or other electronic
structure codes. For example, Entries can be used as inputs for DeepMD-Kit.
"""


class Entry(MSONable):
    """
    An lightweight Entry object containing key computed data
    for storing purpose. 

    """

    def __init__(self, composition, calculator, inputs,
                 data, entry_id=None, attribute=None, tag=None):
        """
        Initializes a Entry.

        Args:
            composition (Composition): Composition of the entry. For
                flexibility, this can take the form of all the typical input
                taken by a Composition, including a {symbol: amt} dict,
                a string formula, and others.
            inputs (dict): An dict of parameters associated with
                the entry. Defaults to None.
            data (dict): An dict of any additional data associated
                with the entry. Defaults to None.
            entry_id (obj): An optional id to uniquely identify the entry.
            attribute: Optional attribute of the entry. This can be used to
                specify that the entry is a newly found compound, or to specify
                a particular label for the entry, or else ... Used for further
                analysis and plotting purposes. An attribute can be anything
                but must be MSONable.
        """
        self.composition = Composition(composition)
        self.calculator  = calculator
        self.inputs = inputs
        self.data = data 
        self.entry_id = entry_id
        self.name = self.composition.reduced_formula
        self.attribute = attribute
        self.tag = tag

    #def __eq__(self,other):
    #    if not self.composition == other.composition:
    #       return False
    #    if not self.calculator == other.calculator:
    #       return False
    #    if not self.inputs == other.inputs:
    #       return False
    #    if not self.data == other.data:
    #       return False
    #    if not self.name == other.name:
    #       return False
    #    if not self.attribute  == other.attribute:
    #       return False
    #    if not self.tag == other.tag:
    #       return False
    #    return True

    @property
    def number_element(self):
        return len(self.composition)

    def __repr__(self):
        output = ["Entry {} - {}".format(self.entry_id, self.composition.formula),
                  "calculator: {}".format(self.calculator) 
                  ]
        return "\n".join(output)

    def __str__(self):
        return self.__repr__()

    @classmethod
    def from_dict(cls, d):
        dec = MontyDecoder()
        return cls(d["composition"], d["calculator"], 
                   inputs={k: dec.process_decoded(v)
                               for k, v in d.get("inputs", {}).items()},
                   data={k: dec.process_decoded(v)
                         for k, v in d.get("data", {}).items()},
                   entry_id=d.get("entry_id", None),
                   attribute=d["attribute"] if "attribute" in d else None,
                   tag=d["tag"] if "tag" in d else None
                   )

    def as_dict(self):
        return {"@module": self.__class__.__module__,
                "@class": self.__class__.__name__,
                "composition": self.composition.as_dict(),
                "calculator": self.calculator,
                "inputs": json.loads(json.dumps(self.inputs,
                                                    cls=MontyEncoder)),
                "data": json.loads(json.dumps(self.data, cls=MontyEncoder)),
                "entry_id": self.entry_id,
                "attribute": self.attribute,
                "tag": self.tag}



from dpgen.dispatcher.Dispatcher import Dispatcher, _split_tasks, JobRecord

class Entity():
    def __init__(self, ip, instance_id, job_handler=None):
        self.ip = ip
        self.instance_id = instance_id
        self.job_handler = job_handler

class DispatcherList():
    def __init__(self, mdata_machine, mdata_resources, nchunks, cloud_resources=None):
        self.mdata_machine = mdata_machine
        self.mdata_resources = mdata_resources
        self.nchunks = nchunks
        self.cloud_resources = cloud_resources
        self.dispatcher_list = list({"dispatcher": None, 
                                     "dispatcher_status": "unallocated",
                                     "entity": None} for ii in range(nchunks))
    # Base
    def init(self):
        for ii in range(self.nchunks):
            self.create(ii)

    # Base
    def run_jobs(self,
                 resources,
                 command,
                 work_path,
                 tasks,
                 group_size,
                 forward_common_files,
                 forward_task_files,
                 backward_task_files,
                 forward_task_deference = True,
                 mark_failure = False,
                 outlog = 'log',
                 errlog = 'err'):
        task_chunks = _split_tasks(tasks, group_size)
        while True:
            if self.check_all_dispatchers_finished():
                break
            ratio_failure = self.mdata_resources["ratio_failure"]
            self.exception_handling(ratio_failure)
            for ii in range(self.nchunks):
                dispatcher_status = self.check_dispatcher_status(ii)
                if dispatcher_status == "unsubmitted":
                    self.dispatcher_list[ii]["job_handler"] = self.dispatcher_list[ii]["dispatcher"].submit_jobs(resources,
                                                                                                                 command,
                                                                                                                 work_path,
                                                                                                                 task_chunks[ii],
                                                                                                                 group_size,
                                                                                                                 forward_common_files,
                                                                                                                 forward_task_files,
                                                                                                                 backward_task_files,
                                                                                                                 forward_task_deference,
                                                                                                                 outlog,
                                                                                                                 errlog)
                    self.dispatcher_list[ii]["dispatcher_status"] == "running"
                elif dispatcher_status == "finished":
                    # to do 
                    # no jobs in queue, delete current machine
                    # else allocate current machine to unalloc dispatcher
                    pass
                elif dispatcher_status == "running":
                    pass
                elif dispatcher_status == "unallocated":
                    # to do: if we can create machine, then make dispatcher
                    #        else pass
                    pass
                elif dispatcher_status == "terminated":
                    pass

    # Derivate
    def create(self, ii):
        '''if jr.json existed and job not finished, use jr.json to rebuild dispatcher
           else create one machine, then make_dispatcher, change status from unallocated to unsubmitted'''
        if not os.path.exists(os.path.join(os.path.abspath(work_path), "jr.%.06d.json" % ii)):
            # to do: create machine, make dispatcher, change status from unallocated to unsubmitted
            pass
        else:
            task_chunks = _split_tasks(tasks, group_size)
            task_chunks_str = ['+'.join(ii) for ii in self.task_chunks]
            task_hashes = [sha1(ii.encode('utf-8')).hexdigest() for ii in task_chunks_str]
            job_record = JobRecord(work_path, task_chunks[ii], fname = "jr.%.06d.json" % ii)
            cur_chunk = self.task_chunks[ii]
            cur_hash = task_hashes[ii]
            if not job_record.check_finished(cur_hash): 
                with open(os.path.join(work_path, fn)) as fp:
                    jr = json.load(fp)
                    self.dispatcher_list[ii]["entity"] = Entity(jr[cur_hash]['context'][3], jr[cur_hash]['context'][4])
                    self.make_dispatcher(ii)
    # Derivate
    def delete(self):
        '''delete one machine'''
        pass

    # Base
    def check_all_dispatchers_finished(self, ratio_failure=0):
        exception_num = 0
        for ii in range(self.nchunks):
            if self.dispatcher_list[ii]["dispatcher_status"] == "terminated":
                exception_num += 1
        if exception_num / self.nchunks > ratio_failure: return False
        else: return True

    # Base
    def exception_handling(self, ratio_failure):
        terminated_num = 0
        for ii in range(self.nchunks):
            if self.dispatcher_list[ii]["dispatcher_status"] == "terminated":
                terminated_num += 1
        if terminated_num / self.nchunks > ratio_failure:
            self.resubmit()

    # Derivate
    def resubmit(self):
        '''create machines
           make dispatcher
           change status from terminated to unsubmitted'''
        pass

    # Base
    def make_dispatcher(self, ii):
        '''use entity to distinguish machine, for example if ip isn't None, means we can make_dispatcher
           change status from unallocated to unsubmitted'''
        entity = self.dispatcher_list[ii]["entity"]
        if entity.ip:
            profile = self.mdata_machine.copy()
            profile['hostname'] = entity.ip
            profile['instance_id'] = entity.instance_id
            self.dispatcher_list[ii]["dispatcher"] = Dispatcher(profile, context_type='ssh', batch_type='shell', job_record='jr.%.06d.json' % ii)
            self.dispatcher_list[ii]["dispatcher_status"] = "unsubmitted"

    # Base
    def check_dispatcher_status(self, ii):
        '''catch running dispatcher exception
           if no exception occured, check finished'''
        if self.dispatcher_list[ii]["dispatcher_status"] == "running":
            if not self.catch_dispatcher_exception():
                if self.dispatcher_list[ii]["dispatcher"].all_finished():
                    self.dispatcher_list[ii]["dispatcher_status"] = "finished"
            else:
                self.dispatcher_list[ii]["dispatcher_status"] = "terminated"
        return self.dispatcher_list[ii]["dispatcher_status"]

    # Derivate
    def catch_dispatcher_exception(self):
        pass









#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from custodian.vasp.jobs import VaspJob as cvj
from custodian.vasp.validators import VaspFilesValidator,VasprunXMLValidator
from custodian.vasp.handlers import VaspErrorHandler,UnconvergedErrorHandler, \
        NonConvergingErrorHandler,FrozenJobErrorHandler,StdErrHandler,\
        WalltimeHandler,PositiveEnergyErrorHandler
from custodian import Custodian
import argparse

handlers=[VaspErrorHandler(),FrozenJobErrorHandler(),StdErrHandler(),NonConvergingErrorHandler(),
           WalltimeHandler(),PositiveEnergyErrorHandler(),UnconvergedErrorHandler()]
validators=[VaspFilesValidator(),VasprunXMLValidator()]

def runvasp(cmd,opt=False,max_errors=3,backup=False,auto_gamma=False,auto_npar=False,ediffg=-.05):
    """
    cmd example:
    cmd=['mpirun', '-np', '32' , '-machinefile', 'hosts','vasp_std']
    """
    if opt:
       jobs = cvj.full_opt_run(cmd, auto_npar=auto_npar, ediffg=ediffg, backup=backup, auto_gamma=auto_gamma )
    else:
       jobs =  [cvj(cmd, auto_npar=auto_npar, backup=backup, auto_gamma=auto_gamma)]
    c = Custodian(handlers, jobs, validators=validators,max_errors=max_errors)
    c.run()

def __main():
    parser = argparse.ArgumentParser()
    parser.add_argument("CMD", type=str,
                        help="""The command for runing vasp, e.g.,
                              'mpirun -np 32 /path/vasp_std' or
                              'srun /path/vasp_std'
                             """)
    parser.add_argument("MAXERR", type=int,
                        help="The maximum error time for runing vasp")
    args = parser.parse_args()
    cmd=args.CMD.split()
    runvasp(cmd=cmd,max_errors=args.MAXERR)
    
if __name__=='__main__':
   __main()
   #vasp="/sharedext4/vasp/vasp.5.4.4/bin/vasp_std"
   #runvasp(cmd=['srun', vasp],max_errors=3,backup=True)
   #runvasp(cmd=['mpirun', '-np', ncpu, fp_cmd],max_errors=max_errors)



import sys,os
sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), '..', '..')))
from dpgen.auto_test.lib.vasp import *

def setUpModule():
    os.chdir(os.path.abspath(os.path.dirname(__file__)))



#!/usr/bin/env python3

import os, re, argparse, filecmp, json, glob, sys
import subprocess as sp
import numpy as np
import dpgen.auto_test.lib.vasp as vasp
import dpgen.auto_test.lib.lammps as lammps
import dpgen.auto_test.lib.util as util
from pymatgen.analysis.elasticity.elastic import ElasticTensor
from pymatgen.analysis.elasticity.strain import Strain
from pymatgen.analysis.elasticity.stress import Stress

global_equi_name = '00.equi'
global_task_name = '02.elastic'

def result_et(et,conf_dir,result):
    with open(result,'w') as fp:
        fp.write('conf_dir:%s\n'% (conf_dir))
        for ii in range(6) :
            for jj in range(6) :
                fp.write ("%7.2f " % (et.voigt[ii][jj] / 1e4))
            fp.write('\n')
        BV = et.k_voigt / 1e4
        GV = et.g_voigt / 1e4
        EV = 9*BV*GV/(3*BV+GV)
        uV = 0.5*(3*BV-2*GV)/(3*BV+GV)
        fp.write("# Bulk   Modulus BV = %.2f GPa\n" % (BV))
        fp.write("# Shear  Modulus GV = %.2f GPa\n" % (GV))
        fp.write("# Youngs Modulus EV = %.2f GPa\n" % (EV))
        fp.write("# Poission Ratio uV = %.2f \n" % (uV))
    fp.close()


def print_et (et):
    for ii in range(6) :
        for jj in range(6) :
            sys.stdout.write ("%7.2f " % (et.voigt[ii][jj] / 1e4))
        sys.stdout.write('\n')
    BV = et.k_voigt / 1e4
    GV = et.g_voigt / 1e4
    EV = 9*BV*GV/(3*BV+GV)
    uV = 0.5*(3*BV-2*GV)/(3*BV+GV)
    print("# Bulk   Modulus BV = %.2f GPa" % (BV))
    print("# Shear  Modulus GV = %.2f GPa" % (GV))
    print("# Youngs Modulus EV = %.2f GPa" % (EV))
    print("# Poission Ratio uV = %.2f " % (uV))

def cmpt_vasp(jdata, conf_dir) :
    conf_path = os.path.abspath(conf_dir)
    conf_poscar = os.path.join(conf_path, 'POSCAR')
    task_path = re.sub('confs', global_task_name, conf_path)
    if 'relax_incar' in jdata.keys():
        vasp_str='vasp-relax_incar'
    else:
        fp_params = jdata['vasp_params']
        kspacing = fp_params['kspacing']
        vasp_str='vasp-k%.2f' % kspacing
    task_path = os.path.join(task_path, vasp_str)

    equi_stress = Stress(np.loadtxt(os.path.join(task_path, 'equi.stress.out')))

    lst_dfm_path = glob.glob(os.path.join(task_path, 'dfm-*'))
    lst_strain = []
    lst_stress = []
    for ii in lst_dfm_path :
        strain = np.loadtxt(os.path.join(ii, 'strain.out'))
        stress = vasp.get_stress(os.path.join(ii, 'OUTCAR'))
        # convert from pressure in kB to stress
        stress *= -1000
        lst_strain.append(Strain(strain))
        lst_stress.append(Stress(stress))
    et = ElasticTensor.from_independent_strains(lst_strain, lst_stress, eq_stress = equi_stress, vasp = False)
    # et = ElasticTensor.from_independent_strains(lst_strain, lst_stress, eq_stress = None)
    # bar to GPa
    # et = -et / 1e4
    print_et(et)
    result = os.path.join(task_path,'result')
    result_et(et,conf_dir,result)
    if 'upload_username' in jdata.keys():
        upload_username=jdata['upload_username']
        util.insert_data('elastic','vasp',upload_username,result)


def cmpt_deepmd_lammps(jdata, conf_dir, task_name) :
    conf_path = os.path.abspath(conf_dir)
    conf_poscar = os.path.join(conf_path, 'POSCAR')
    task_path = re.sub('confs', global_task_name, conf_path)
    task_path = os.path.join(task_path, task_name)
    equi_stress = Stress(np.loadtxt(os.path.join(task_path, 'equi.stress.out')))

    lst_dfm_path = glob.glob(os.path.join(task_path, 'dfm-*'))
    lst_strain = []
    lst_stress = []
    for ii in lst_dfm_path :
        strain = np.loadtxt(os.path.join(ii, 'strain.out'))
        stress = lammps.get_stress(os.path.join(ii, 'log.lammps'))
        # convert from pressure to stress
        stress = -stress
        lst_strain.append(Strain(strain))
        lst_stress.append(Stress(stress))
    et = ElasticTensor.from_independent_strains(lst_strain, lst_stress, eq_stress = equi_stress, vasp = False)
    # et = ElasticTensor.from_independent_strains(lst_strain, lst_stress, eq_stress = None)
    # bar to GPa
    # et = -et / 1e4
    print_et(et)
    result = os.path.join(task_path,'result')
    result_et(et,conf_dir,result)
    if 'upload_username' in jdata.keys() and task_name=='deepmd':
        upload_username=jdata['upload_username']
        util.insert_data('elastic','deepmd',upload_username,result)



def _main() :
    parser = argparse.ArgumentParser(
        description="cmpt 02.elastic")
    parser.add_argument('TASK', type=str,
                        help='the task of generation, vasp or lammps')
    parser.add_argument('PARAM', type=str,
                        help='json parameter file')
    parser.add_argument('CONF', type=str,
                        help='the path to conf')
    args = parser.parse_args()

    with open (args.PARAM, 'r') as fp :
        jdata = json.load (fp)

    print('# generate %s task with conf %s' % (args.TASK, args.CONF))
    if args.TASK == 'vasp':
        cmpt_vasp(jdata, args.CONF)
    elif args.TASK == 'deepmd' :
        cmpt_deepmd_lammps(jdata, args.CONF, args.TASK)
    elif args.TASK == 'meam' :
        cmpt_deepmd_lammps(jdata, args.CONF, args.TASK)
    else :
        raise RuntimeError("unknow task ", args.TASK)

if __name__ == '__main__' :
    _main()



#!/usr/bin/env python3

import os, re, argparse
import dpgen.auto_test.lib.crys as crys
from pymatgen.ext.matproj import MPRester, Composition
from pymatgen.analysis.structure_matcher import StructureMatcher

global_std_crystal = {
    'fcc' : crys.fcc,
    'hcp' : crys.hcp,
    'dhcp' : crys.dhcp,
    'bcc' : crys.bcc,
    'diamond' : crys.diamond,
    'sc' : crys.sc
}

def test_fit(struct, data) :
    m = StructureMatcher() 
    for ii in data :
        if m.fit(ii['structure'], struct) :
            return True
    return False

def make_path_mp(ii) :
    pf = ii['pretty_formula']
    pf = re.sub('\d+', '', pf)
    task_id = ii['task_id']
    work_path = 'confs'
    work_path = os.path.join(work_path, pf)
    work_path = os.path.join(work_path, task_id)
    return work_path

def gen_ele_std(ele_name, ctype):
    struct = global_std_crystal[ctype](ele_name)
    work_path = 'confs'
    work_path = os.path.join(work_path, ele_name)
    work_path = os.path.join(work_path, 'std-'+ctype)
    os.makedirs(work_path, exist_ok = True)
    fposcar = os.path.join(work_path, 'POSCAR')
    fjson = os.path.join(work_path, 'data.json')
    struct.to('poscar', fposcar)    
    return struct

def gen_element(ele_name,key) :
    assert(type(ele_name) == str)
    mpr = MPRester(key)
    data = mpr.query({'elements':[ele_name], 'nelements':1}, 
                     properties=["task_id", 
                                 "pretty_formula", 
                                 'formula', 
                                 "anonymous_formula",                             
                                 'formation_energy_per_atom',
                                 'energy_per_atom',
                                 'structure'])
    for ii in data :
        work_path = make_path_mp(ii)
        os.makedirs(work_path, exist_ok = True)
        fposcar = os.path.join(work_path, 'POSCAR')
        fjson = os.path.join(work_path, 'data.json')
        ii['structure'].to('poscar', fposcar)
        ii['structure'].to('json', fjson)

    m = StructureMatcher() 
    for ii in global_std_crystal.keys() :
        ss = gen_ele_std(ele_name, ii)
        find = False
        for jj in data:
            if m.fit(ss,jj['structure']) :
                find = True
                break
        if find :            
            work_path = make_path_mp(jj)
            with open(os.path.join(work_path,'std-crys'), 'w') as fp :
                fp.write(ii+'\n')

def gen_element_std(ele_name) :
    assert(type(ele_name) == str)
    for ii in global_std_crystal.keys() :
        ss = gen_ele_std(ele_name, ii)

def gen_alloy(eles,key) :
    
    mpr = MPRester(key)

    data = mpr.query({'elements':{'$all': eles}, 'nelements':len(eles)}, 
                     properties=["task_id", 
                                 "pretty_formula", 
                                 'formula', 
                                 "anonymous_formula",                             
                                 'formation_energy_per_atom',
                                 'energy_per_atom',
                                 'structure'])
    if len(data) == 0 :
        return
    
    alloy_file = make_path_mp(data[0])
    os.makedirs(alloy_file, exist_ok = True)
    alloy_file = os.path.join(alloy_file, '..')
    alloy_file = os.path.join(alloy_file, 'alloy')
    with open(alloy_file, 'w') as fp :
        None    
    
    for ii in data :
        work_path = make_path_mp(ii)
        os.makedirs(work_path, exist_ok = True)
        fposcar = os.path.join(work_path, 'POSCAR')
        fjson = os.path.join(work_path, 'data.json')
        ii['structure'].to('poscar', fposcar)
        ii['structure'].to('json', fjson)

def _main() :
    parser = argparse.ArgumentParser(
        description="gen structures")
    parser.add_argument('key', type=str,
                        help='key id of material project')
    parser.add_argument('elements', 
                        type=str,
                        nargs = '+',
                        help="the list of appeared elements")
    args = parser.parse_args()

    print('generate %s' % (args.elements))
    if len(args.elements) == 1 :
        gen_element(args.elements[0],key)
        # gen_element_std(args.elements[0])
    else :
        gen_alloy(args.elements,key)

if __name__ == '__main__' :
    _main()




import os,sys,json,glob,shutil
import dpdata
import numpy as np
import unittest

sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), '..')))
__package__ = 'generator'
from .context import check_bad_box

class TestCheckBadBox(unittest.TestCase):
    def test_length_ratio(self):
        dirname = os.path.dirname(__file__)
        conf_bad = os.path.join(dirname, 'check_bad_box', 'bad.length.lammpstrj')
        conf_good = os.path.join(dirname, 'check_bad_box', 'good.lammpstrj')
        self.assertTrue(check_bad_box(conf_bad, 'length_ratio:5'))
        self.assertFalse(check_bad_box(conf_good, 'length_ratio:5'))

    def test_height_ratio(self):
        dirname = os.path.dirname(__file__)
        conf_bad = os.path.join(dirname, 'check_bad_box', 'bad.height.POSCAR')
        self.assertTrue(check_bad_box(conf_bad, 'height_ratio:5', fmt = 'vasp/POSCAR'))
        self.assertFalse(check_bad_box(conf_bad, 'length_ratio:5', fmt = 'vasp/POSCAR'))



import os,sys,json,glob,shutil
import unittest
from pymatgen import Structure,Composition

sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), '..')))
__package__ = 'data'
from .context import setUpModule
from .context_bulk import *

class TestGenBulk(unittest.TestCase):
    def setUp(self):
        self.alloy=[] 
        with open (param_file, 'r') as fp :
            jdata = json.load (fp)
        out_dir = out_dir_name(jdata)
        self.out_dir= out_dir
        jdata['out_dir'] = out_dir
        self.elements=jdata["elements"]
        self.scale_numb=len(jdata["scale"])
        self.pert_numb=jdata["pert_numb"]
        self.root_dir= out_dir
        create_path(out_dir)
        make_unit_cell(jdata)
        make_super_cell(jdata)
        place_element(jdata)
        make_vasp_relax(jdata,{"fp_resources":{}})
        make_scale(jdata)
        pert_scaled(jdata)

    def tearDown(self):
        shutil.rmtree(self.root_dir)

    def test(self):
        path=self.out_dir+"/00.place_ele"
        struct0=Structure.from_file(os.path.join(path,"POSCAR"))
        alloys=glob.glob(os.path.join(path,"sys-*"))
        self.assertEqual(len(alloys),len(struct0)-1)
        for ii in alloys:
            elem_numb=[int(i) for i in ii.split('/')[-1].split('-')[1:]]
            comp=''
            for num, el in zip(elem_numb,self.elements):
                comp+=el+str(num)
            comp=Composition(comp)
            struct=Structure.from_file(os.path.join(ii,"POSCAR"))
            self.assertEqual(struct.composition,comp) 
        path=self.out_dir+"/01.scale_pert"
        alloys=glob.glob(os.path.join(path,"sys-*"))
        self.assertEqual(len(alloys),len(struct0)-1)
        for ii in alloys:
            scales=glob.glob(os.path.join(ii,"scale-*"))
            self.assertEqual(len(scales),self.scale_numb)
            for scale in scales:
                perts=glob.glob(os.path.join(scale,"[0-9]*")) 
                self.assertEqual(len(perts),self.pert_numb+1)



import os,shutil,uuid
import subprocess as sp
from glob import glob
from dpgen import dlog

class SPRetObj(object) :
    def __init__ (self,
                  ret) :
        self.data = ret

    def read(self) :
        return self.data

    def readlines(self) :
        lines = self.data.decode('utf-8').splitlines()
        ret = []
        for aa in lines:
            ret.append(aa+'\n')
        return ret

class LazyLocalContext(object) :
    def __init__ (self,
                  local_root,
                  work_profile = None,
                  job_uuid = None) :
        """
        work_profile:
        local_root:
        """
        assert(type(local_root) == str)
        self.local_root = os.path.abspath(local_root)
        self.remote_root = self.local_root
        if job_uuid:
           self.job_uuid=job_uuid
        else:
           self.job_uuid = str(uuid.uuid4())
        
    def get_job_root(self) :
        return self.local_root

    def upload(self,
               job_dirs,
               local_up_files,
               dereference = True) :
        pass

    def download(self, 
                 job_dirs,
                 remote_down_files,
                 check_exists = False,
                 mark_failure = True,
                 back_error=False) :
        for ii in job_dirs :
            for jj in remote_down_files :
                fname = os.path.join(self.local_root, ii, jj)
                exists = os.path.exists(fname)
                if not exists:
                    if check_exists:
                        if mark_failure:
                            with open(os.path.join(self.local_root, ii, 'tag_failure_download_%s' % jj), 'w') as fp: pass
                        else:
                            pass
                    else:
                        raise RuntimeError('do not find download file ' + fname)


    def block_checkcall(self,
                        cmd) :
        cwd = os.getcwd()
        os.chdir(self.local_root)
        proc = sp.Popen(cmd, shell=True, stdout = sp.PIPE, stderr = sp.PIPE)
        o, e = proc.communicate()
        stdout = SPRetObj(o)
        stderr = SPRetObj(e)
        code = proc.returncode
        if code != 0:
            os.chdir(cwd)        
            raise RuntimeError("Get error code %d in locally calling %s with job: %s ", (code, cmd, self.job_uuid))
        os.chdir(cwd)        
        return None, stdout, stderr
        
    def block_call(self, cmd) :
        cwd = os.getcwd()
        os.chdir(self.local_root)
        proc = sp.Popen(cmd, shell=True, stdout = sp.PIPE, stderr = sp.PIPE)
        o, e = proc.communicate()
        stdout = SPRetObj(o)
        stderr = SPRetObj(e)
        code = proc.returncode
        os.chdir(cwd)        
        return code, None, stdout, stderr

    def clean(self) :
        pass

    def write_file(self, fname, write_str):
        with open(os.path.join(self.local_root, fname), 'w') as fp :
            fp.write(write_str)

    def read_file(self, fname):
        with open(os.path.join(self.local_root, fname), 'r') as fp:
            ret = fp.read()
        return ret

    def check_file_exists(self, fname):
        return os.path.isfile(os.path.join(self.local_root, fname))
        
    def call(self, cmd) :
        cwd = os.getcwd()
        os.chdir(self.local_root)
        proc = sp.Popen(cmd, shell=True, stdout = sp.PIPE, stderr = sp.PIPE)
        os.chdir(cwd)        
        return proc

    def kill(self, proc):
        proc.kill()

    def check_finish(self, proc):
        return (proc.poll() != None)

    def get_return(self, proc):
        ret = proc.poll()
        if ret is None:
            return None, None, None
        else :
            try:
                o, e = proc.communicate()
                stdout = SPRetObj(o)
                stderr = SPRetObj(e)
            except:
                stdout = None
                stderr = None
        return ret, stdout, stderr
    
    
